/***************************************************************************
 *            IpFilterManager.h
 *
 *  Copyright  2008  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#ifndef IPFILTERMANAGER_H
#define IPFILTERMANAGER_H

#include <string>
#include <functional>
#include <algorithm>

#include <boost/signal.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/filesystem.hpp>
#include <boost/filesystem/fstream.hpp>

#include "libtorrent/ip_filter.hpp"

namespace fs = boost::filesystem;

// TODO: FINISH ipv6 support
namespace Lince
{
	
class IpFilterManager;
typedef boost::shared_ptr<IpFilterManager> IpFilterManagerPtr;
typedef std::vector<libtorrent::ip_range<libtorrent::address_v4> > ips_v4;
typedef std::vector<libtorrent::ip_range<libtorrent::address_v6> > ips_v6;
	
typedef boost::signal<void (std::string)> stringSignal;

class IpFilterManager : public libtorrent::ip_filter 
{	
	// Emule's format
	void read_ipv4_file(const fs::path &);
	void save_ipv4_file(const fs::path &);
	
	stringSignal m_signal_ip_banned;
	stringSignal m_signal_ip_unbanned;
	
public:
	
	IpFilterManager();
		
	// Ipfilter management
	ips_v4 get_ips_v4() const
	{
		return export_filter().get<0>();
	}
		
	ips_v4 get_filtered_ips_v4() const;
		
	ips_v6 get_ips_v6() const
	{
		return export_filter().get<1>();
	}
	
	void add_ipv4_rule(const std::string &, int);
	void add_ipv4_rule(const libtorrent::address_v4 &, const libtorrent::address_v4 &, int);
		
	bool load_from_file_v4(const std::string &);
	bool load_from_file_v4(const fs::path &);
		
	bool save_to_file_v4(const std::string &);
	bool save_to_file_v4(const fs::path &);
		
	boost::signals::connection ip_banned_signal(stringSignal::slot_function_type subscriber)
	{
		return m_signal_ip_banned.connect(subscriber);
	};
		
	boost::signals::connection ip_unbanned_signal(stringSignal::slot_function_type subscriber)
	{
		return m_signal_ip_unbanned.connect(subscriber);
	};
		
	~IpFilterManager();
	
};

} // Namespace

#endif
