/***************************************************************************
 *            XMLReader.h
 *
 *  Copyright  2008  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */
 
#ifndef _XMLREADER_H
#define _XMLREADER_H

extern "C"
{
#include <libxml/xmlreader.h>
}

#include <string>

#define MAXVALUE 20


class XMLReader{
	public:
		XMLReader();
		XMLReader(const std::string &);
		~XMLReader();
	
		void setFileName(const std::string & fname) { filename = fname; };
		const std::string & getFileName(void) const { return filename; };
				
		int goToNextElement(void);
		int getDepth(void) const { return xmlTextReaderDepth(reader); }
		
		bool goToRootElement(void);
		bool goToNextAttribute(void);
		bool findChild(const std::string &);
		bool findChildFromBeginning(const std::string &);
		bool isEmpty(void) { return xmlTextReaderIsEmptyElement(reader); };
		bool hasAttributes(void) { return (numattrs > 0); };
	
		const std::string & getNodeName(void) const { return presentnode; };
		const std::string & getFather(void) const { return father_node[getDepth() - 1]; };
		std::string getChildData(void);
		const std::string & getAttributeData(void) const { return attributedata; };
		const std::string & getAttributeName(void) const { return attributename; };
		
	private:
		bool initializeReader(void);
		bool isReaderInitilizated(void) const { return !(reader == NULL); };
		void freeReader(void) { xmlFreeTextReader(reader); reader = NULL; };
		
		std::string filename;
		
		int numattrs, currentattr;
		xmlTextReaderPtr reader;
		std::string childdata;
		std::string attributename;
		std::string attributedata;
		std::string presentnode;
		std::string father_node[MAXVALUE], attributedatatmp[MAXVALUE];
};

#endif /* _XMLREADER_H */
