//           OptionsTab.cc
//  Sat Mar 31 01:27:01 2007
//  Copyright  2007  Fernando Tarín Morales
//  icemanf@gmail.com

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA

#include "OptionsTab.h"
#include "LogTab.h"
#include "../config.h"
#include <glibmm/i18n.h>

#include "Lince/Core.h"
#include "Lince/SetupManager.h"
#include "Lince/SessionManager.h"
#include "Lince/TorrentsManager.h"

#include "NoteBookManager.h"

#include <iostream>
#include <glibmm.h>
#include <boost/bind.hpp>

using namespace Lince;

OptionsTab::OptionsTab(NoteBookManager * _nb):
	cw(this),
	pw(this),
	ew(this),
	tw(this),
	scw(this),
	fw(this),
	tow(this),
	sw(this),
	lw(this),
	nw(this),
	qw(this),
	ifw(this),
	shw(this),
	BaseTab(_("Options"))
{
	optionsWindow.add(optionsTreeView);

	// Only show the scrollbars when they are necessary:
	optionsWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	refTreeModel = Gtk::TreeStore::create(m_Columns);
  	optionsTreeView.set_model(refTreeModel);

	Gtk::TreeModel::Row row, childrow;

	//Fill the TreeView's model
  	row = *(refTreeModel->append());
	row[m_Columns.m_col_id] = 1;
  	row[m_Columns.m_col_name] = _("Connection");

  	childrow = *(refTreeModel->append(row.children()));
	childrow[m_Columns.m_col_id] = 2;
  	childrow[m_Columns.m_col_name] = _("Proxy settings");

	childrow = *(refTreeModel->append(row.children()));
	childrow[m_Columns.m_col_id] = 3;
  	childrow[m_Columns.m_col_name] = _("Encryption settings");

	row = *(refTreeModel->append());
	row[m_Columns.m_col_id] = 4;
  	row[m_Columns.m_col_name] = _("Transference");

	childrow = *(refTreeModel->append(row.children()));
	childrow[m_Columns.m_col_id] = 5;
  	childrow[m_Columns.m_col_name] = _("Scheduler");

	row = *(refTreeModel->append());
	row[m_Columns.m_col_id] = 6;
  	row[m_Columns.m_col_name] = _("Files");

	childrow = *(refTreeModel->append(row.children()));
	childrow[m_Columns.m_col_id] = 7;
  	childrow[m_Columns.m_col_name] = "Torrents";

	row = *(refTreeModel->append());
	row[m_Columns.m_col_id] = 8;
  	row[m_Columns.m_col_name] = _("Statistics");

	row = *(refTreeModel->append());
	row[m_Columns.m_col_id] = 9;
  	row[m_Columns.m_col_name] = _("Log");

  	row = *(refTreeModel->append());
  	row[m_Columns.m_col_id] = 10;
  	row[m_Columns.m_col_name] = _("Notifications");

	row = *(refTreeModel->append());
	row[m_Columns.m_col_id] = 11;
  	row[m_Columns.m_col_name] = _("Queue");

	row = *(refTreeModel->append());
	row[m_Columns.m_col_id] = 12;
  	row[m_Columns.m_col_name] = _("Ip filter");

	row = *(refTreeModel->append());
	row[m_Columns.m_col_id] = 13;
  	row[m_Columns.m_col_name] = _("Search");


	//Add the TreeView's view columns:
	optionsTreeView.append_column(_("Name"), m_Columns.m_col_name);

	for (int i=FIRST_WINDOW;i<LAST_WINDOW;i++) windowsUsed[i] = true;

	configWindowsBox.pack_start(cw, true, true);
	configWindowsBox.pack_start(pw, true, true);
	configWindowsBox.pack_start(ew, true, true);
	configWindowsBox.pack_start(tw, true, true);
	configWindowsBox.pack_start(scw, true, true);
	configWindowsBox.pack_start(fw, true, true);
	configWindowsBox.pack_start(tow, true, true);
	configWindowsBox.pack_start(sw, true, true);
	configWindowsBox.pack_start(lw, true, true);
	configWindowsBox.pack_start(nw, true, true);
	configWindowsBox.pack_start(qw, true, true);
	configWindowsBox.pack_start(ifw, true, true);
	configWindowsBox.pack_start(shw, true, true);

	pane.pack1(optionsWindow, false, true);
	pane.pack2(configWindowsBox, false, false);

	pane.set_position(130);

	// Signal
	Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = optionsTreeView.get_selection();
	refTreeSelection->signal_changed().connect( sigc::mem_fun(*this, &OptionsTab::on_treeview_row_changed) );

	// Gui Setup Manager
	gsm = NULL;
	nb = _nb;

	Core::get_alerts_manager()->ban_peer_signal(
			boost::bind(&OptionsTab::banIP, this, _1));
}

OptionsTab::~OptionsTab()
{
}

void OptionsTab::emptyWindowsBox()
{
	int i;

	for (i=FIRST_WINDOW;i<LAST_WINDOW;i++)
	{
		if (windowsUsed[i])
		{
			if (i == CONNECTION) configWindowsBox.remove(cw);
			if (i == PROXY) configWindowsBox.remove(pw);
			if (i == ENCRYPTION) configWindowsBox.remove(ew);
			if (i == TRANSFERENCE) configWindowsBox.remove(tw);
			if (i == SCHEDULER) configWindowsBox.remove(scw);
			if (i == FILES) configWindowsBox.remove(fw);
			if (i == TORRENTS) configWindowsBox.remove(tow);
			if (i == STATISTICS) configWindowsBox.remove(sw);
			if (i == LOG) configWindowsBox.remove(lw);
			if (i == NOTIFICATIONS) configWindowsBox.remove(nw);
			if (i == QUEUE) configWindowsBox.remove(qw);
			if (i == IP_FILTER) configWindowsBox.remove(ifw);
			if (i == SEARCH) configWindowsBox.remove(shw);
			windowsUsed[i] = false;
		}
	}
}


void OptionsTab::on_treeview_row_changed()
{
	Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = optionsTreeView.get_selection();
	Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();

	if(iter) //If anything is selected
	{
		Gtk::TreeModel::Row row = *iter;
		emptyWindowsBox();
		if (row[m_Columns.m_col_id] == 1)
		{
			configWindowsBox.pack_start(cw, true , true);
			windowsUsed[CONNECTION] = true;
		}
		if (row[m_Columns.m_col_id] == 2)
		{
			configWindowsBox.pack_start(pw, true, true);
			windowsUsed[PROXY] = true;
		}
		if (row[m_Columns.m_col_id] == 3)
		{
			configWindowsBox.pack_start(ew, true, true);
			windowsUsed[ENCRYPTION] = true;
		}
		if (row[m_Columns.m_col_id] == 4)
		{
			configWindowsBox.pack_start(tw, true, true);
			windowsUsed[TRANSFERENCE] = true;
		}
		if (row[m_Columns.m_col_id] == 5)
		{
			configWindowsBox.pack_start(scw, true, true);
			windowsUsed[SCHEDULER] = true;
		}
		if (row[m_Columns.m_col_id] == 6)
		{
			configWindowsBox.pack_start(fw, true, true);
			windowsUsed[FILES] = true;
		}
		if (row[m_Columns.m_col_id] == 7)
		{
			configWindowsBox.pack_start(tow, true, true);
			windowsUsed[TORRENTS] = true;
		}
		if (row[m_Columns.m_col_id] == 8){
			configWindowsBox.pack_start(sw, true, true);
			windowsUsed[STATISTICS] = true;
		}
		if (row[m_Columns.m_col_id] == 9)
		{
			configWindowsBox.pack_start(lw, true, true);
			windowsUsed[LOG] = true;
		}
		if (row[m_Columns.m_col_id] == 10)
		{
			configWindowsBox.pack_start(nw, true, true);
			windowsUsed[NOTIFICATIONS] = true;
		}
		if (row[m_Columns.m_col_id] == 11)
		{
			configWindowsBox.pack_start(qw, true, true);
			windowsUsed[QUEUE] = true;
		}
		if (row[m_Columns.m_col_id] == 12)
		{
			configWindowsBox.pack_start(ifw, true, true);
			windowsUsed[IP_FILTER] = true;
		}
		if (row[m_Columns.m_col_id] == 13)
		{
			configWindowsBox.pack_start(shw, true, true);
			windowsUsed[SEARCH] = true;
		}
	}
}

void OptionsTab::setConnectionWindow()
{
	/*emptyWindowsBox();
	configWindowsBox.pack_start(cw, true , true);
	windowsUsed[CONNECTION] = true;*/

	std::ostringstream rowpos;
    rowpos << 0;
    optionsTreeView.set_cursor(Gtk::TreePath(rowpos.str()));
}

void OptionsTab::setGuiSetupManager(GuiSetupManager * _gsm)
{
	gsm = _gsm;
	updateOptionsTab();
};


void OptionsTab::updateOptionsTab()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();

	// Connection Settings
	cw.setLastTcpButtonData(csm->get(SetupManager::FIRST_PORT));
	cw.setFirstTcpButtonData(csm->get(SetupManager::FIRST_PORT));
	cw.setDhtActivateButtonData(csm->get(SetupManager::DHT_SUPPORT));
	cw.setUpnpActivateButtonData(csm->get(SetupManager::UPNP_SUPPORT));
	cw.setLocalDiscoveryActivateButtonData(csm->get(SetupManager::LOCAL_DISCOVERY_SUPPORT));
	cw.setMetadataActivateButtonData(csm->get(SetupManager::METADATA_SUPPORT));
	cw.setPexActivateButton(csm->get(SetupManager::PEX_SUPPORT));
	cw.setSmartBanButton(csm->get(SetupManager::SMARTBAN_SUPPORT));
	cw.setAdvancedBindToInterfaceEntry(csm->get(SetupManager::BIND_TO_INTERFACE));
	cw.setAdvancedMaxHalfOpenConnectionsButton(csm->get(SetupManager::MAX_HALF_OPEN_CONNECTIONS));

	// Proxy Settings
	pw.setServerProxyNameEntry(csm->get(SetupManager::SERVER_PROXY_NAME));
	pw.setServerProxyPortButton(csm->get(SetupManager::SERVER_PROXY_PORT));
	pw.setServerProxyLoginEntry(csm->get(SetupManager::SERVER_PROXY_USERNAME));
	pw.setServerProxyPassEntry(csm->get(SetupManager::SERVER_PROXY_PASSWORD));
	pw.setServerProxyEnableCheckButton(csm->get(SetupManager::SERVER_USE_PROXY));
	pw.setServerProxyType(csm->get(SetupManager::SERVER_PROXY_TYPE));

	pw.setClientProxyNameEntry(csm->get(SetupManager::CLIENT_PROXY_NAME));
	pw.setClientProxyPortButton(csm->get(SetupManager::CLIENT_PROXY_PORT));
	pw.setClientProxyLoginEntry(csm->get(SetupManager::CLIENT_PROXY_USERNAME));
	pw.setClientProxyPassEntry(csm->get(SetupManager::CLIENT_PROXY_PASSWORD));
	pw.setClientProxyEnableCheckButton(csm->get(SetupManager::CLIENT_USE_PROXY));
	pw.setClientProxyType(csm->get(SetupManager::CLIENT_PROXY_TYPE));

	// Encryption Settings
	ew.setEncryptionEnableCheckButton(csm->get(SetupManager::USE_ENCRYPTION));
	ew.setPreferRC4(csm->get(SetupManager::ENCRYPTION_PREFER_RC4));
	ew.setInconmingConnectionsButtonData(csm->get(SetupManager::ENCRYPTED_INCOMING_CONNECTIONS));
	ew.setOutgoingConnectionsButtonData(csm->get(SetupManager::ENCRYPTED_OUTGOING_CONNECTIONS));
	ew.setAllowedEncryptionButtonData(csm->get(SetupManager::ENCRYPTION_LEVEL));

	// Transference Settings
	tw.setMaxUploadButton(csm->get(SetupManager::UPLOAD_SPEED));
	tw.setMaxUploadCheckButton(csm->get(SetupManager::DISABLE_UPLOAD_SPEED));
	tw.setMaxDownloadButton(csm->get(SetupManager::DOWNLOAD_SPEED));
	tw.setMaxDownloadCheckButton(csm->get(SetupManager::DISABLE_DOWNLOAD_SPEED));
	tw.setMaxConnectsButton(csm->get(SetupManager::CONNECTIONS_NUMBER));
	tw.setMaxConnectsCheckButton(csm->get(SetupManager::DISABLE_CONNECTIONS_NUMBER));
	tw.setMaxUploadSlotsButton(csm->get(SetupManager::UPLOAD_SLOTS));
	tw.setMaxUploadSlotsCheckButton(csm->get(SetupManager::DISABLE_UPLOAD_SLOTS));
	tw.setShareRatioButton(csm->get(SetupManager::SHARE_RATIO_1), csm->get(SetupManager::SHARE_RATIO_2));
	tw.setShareRatioCheckButton(csm->get(SetupManager::DISABLE_SHARE_RATIO));
	tw.setAllowConnectionsFromTheSameIp(csm->get(SetupManager::CONNECTIONS_FROM_THE_SAME_IP));
	tw.setPrioritizeFirstPartOfFiles(csm->get(SetupManager::PRIORITIZE_FIRST_PART_OF_FILES));

	// FilesWindow Settings
	fw.setDirectoryEntry(gsm->get(GuiSetupManager::DEFAULT_FILE_DIRECTORY));
	fw.setDownloadDefaultDirectoryCheckButton(gsm->get(GuiSetupManager::USE_DEFAULT_FILE_DIRECTORY));
	fw.setStorageType(csm->get(SetupManager::STORAGE_TYPE));
	//fw.setFastResumeCheckButton(gsm->get(GuiSetupManager::USE_FAST_RESUME));
	//fw.setFastResumeTimeUpdateButton(gsm->get(GuiSetupManager::FAST_RESUME_TIME));
	fw.setConfirmDataEliminationCheckButton(gsm->get(GuiSetupManager::CONFIRM_DATA_ELIMINATION));
	fw.setAskWhenLoadingTorrentsUsingDbusRequestsCheckButton(gsm->get(GuiSetupManager::ASK_WHEN_LOADING_FROM_DBUS_REQUEST));

	// TorrentsOptionsWindow Settings
	tow.setSaveDirectoryEntry(gsm->get(GuiSetupManager::DOWNLOAD_TORRENTS_DIRECTORY));
	tow.setSaveTorrentsCheckButton(gsm->get(GuiSetupManager::USE_DOWNLOAD_TORRENTS_DIRECTORY));
	tow.setRemoveOriginalTorrent(gsm->get(GuiSetupManager::USE_REMOVE_ORIGINAL_TORRENT));
	tow.setRemoveTorrentWhenFinished(gsm->get(GuiSetupManager::USE_REMOVE_TORRENT_WHEN_FINISHED));
	tow.setLoadDirectoryEntry(gsm->get(GuiSetupManager::LOAD_TORRENTS_DIRECTORY));
	tow.setLoadTorrentsCheckButton(gsm->get(GuiSetupManager::USE_LOAD_TORRENTS_DIRECTORY));
	tow.setAllowChangeDirectory(gsm->get(GuiSetupManager::ALLOW_CHANGE_LOAD_DIRECTORY));

	// StatisticsWindow
	sw.setStatisticsTabCheckButton(gsm->get(GuiSetupManager::SHOW_GRAPHIC_STATISTICS));
	sw.setUpdateTimeButton(gsm->get(GuiSetupManager::STATISTICS_TIME_UPDATE));

	// LogWindow
	lw.setActivateLogCheckButton(gsm->get(GuiSetupManager::USE_LOG_TAB));
	lw.setColoredOutputCheckButton(gsm->get(GuiSetupManager::LOG_COLORED_OUTPUT));
	lw.setSaveLogFileCheckButton(gsm->get(GuiSetupManager::SAVE_LOG_TO_FILE));
	lw.setAutoScrollCheckButton(gsm->get(GuiSetupManager::LOG_AUTO_SCROLL));
	lw.setBufferSizeButton(gsm->get(GuiSetupManager::LOG_SIZE));
	lw.setLogLevel(gsm->get(GuiSetupManager::LOG_LEVEL));

	// NotifyWindow
	nw.setActivateNotificationsCheckButton(gsm->getBool(GuiSetupManager::USE_NOTIFICATIONS));
	nw.setNotifyFinishedTorrentsCheckButton(gsm->getBool(GuiSetupManager::NOTIFY_FINISHED_TORRENTS));
	nw.setNotifyStoppedTorrentsCheckButton(gsm->getBool(GuiSetupManager::NOTIFY_STOPPED_TORRENTS));
	nw.setNotifyStartedTorrentsCheckButton(gsm->getBool(GuiSetupManager::NOTIFY_STARTED_TORRENTS));
	nw.setNotifyAddedTorrentsCheckButton(gsm->getBool(GuiSetupManager::NOTIFY_ADDED_TORRENTS));
	nw.setNotifyRemovedTorrentsCheckButton(gsm->getBool(GuiSetupManager::NOTIFY_REMOVED_TORRENTS));
	nw.setNotifyQueuedTorrentsCheckButton(gsm->getBool(GuiSetupManager::NOTIFY_QUEUED_TORRENTS));
	nw.setNotifyUnqueuedTorrentsCheckButton(gsm->getBool(GuiSetupManager::NOTIFY_UNQUEUED_TORRENTS));
	nw.setNotifyShowFromSystrayCheckButton(gsm->getBool(GuiSetupManager::NOTIFY_SHOW_FROM_SYSTRAY));

	// QueueWindow
	qw.setActiveDownloadsQueueButton(csm->get(SetupManager::ACTIVE_DOWNLOADS_VALUE));
	qw.setMaxDownloadsQueueButton(csm->get(SetupManager::MAX_DOWNLOADS_VALUE));
	qw.setActiveSharedQueueButton(csm->get(SetupManager::ACTIVE_SHARES_VALUE));

	// IpFilterWindow
	ifw.setActivateIpFilterCheckButton(csm->get(SetupManager::USE_IP_FILTER));

	// SearchWindow
	shw.setBtjunkieCheckButton(gsm->get(GuiSetupManager::USE_BTJUNKIE));
	shw.setIsohuntCheckButton(gsm->get(GuiSetupManager::USE_ISOHUNT));
	shw.setPiratebayCheckButton(gsm->get(GuiSetupManager::USE_PIRATEBAY));
	shw.setMininovaCheckButton(gsm->get(GuiSetupManager::USE_MININOVA));
	shw.setTorrentreactorCheckButton(gsm->get(GuiSetupManager::USE_TORRENTREACTOR));
	shw.setFilterSeedersCheckButton(gsm->get(GuiSetupManager::FILTER_RESULTS_WITHOUT_SEEDERS));
	shw.setMaxSearchesValue(gsm->get(GuiSetupManager::MAX_NUMBER_OF_SEARCHES));

	// SchedulerWindow
	scw.setUseSchedulerCheckButton(csm->get(SetupManager::USE_SCHEDULER));
	scw.setMondayCheckButton(csm->get(SetupManager::SCHEDULER_MONDAY));
	scw.setTuesdayCheckButton(csm->get(SetupManager::SCHEDULER_TUESDAY));
	scw.setWednesdayCheckButton(csm->get(SetupManager::SCHEDULER_WEDNESDAY));
	scw.setThursdayCheckButton(csm->get(SetupManager::SCHEDULER_THURSDAY));
	scw.setFridayCheckButton(csm->get(SetupManager::SCHEDULER_FRIDAY));
	scw.setSaturdayCheckButton(csm->get(SetupManager::SCHEDULER_SATURDAY));
	scw.setSundayCheckButton(csm->get(SetupManager::SCHEDULER_SUNDAY));
	scw.setDownloadHour0Value(csm->get(SetupManager::SCHEDULER_HOUR0_DOWN));
	scw.setUploadHour0Value(csm->get(SetupManager::SCHEDULER_HOUR0_UP));
	scw.setDownloadHour1Value(csm->get(SetupManager::SCHEDULER_HOUR1_DOWN));
	scw.setUploadHour1Value(csm->get(SetupManager::SCHEDULER_HOUR1_UP));
	scw.setDownloadHour2Value(csm->get(SetupManager::SCHEDULER_HOUR2_DOWN));
	scw.setUploadHour2Value(csm->get(SetupManager::SCHEDULER_HOUR2_UP));
	scw.setDownloadHour3Value(csm->get(SetupManager::SCHEDULER_HOUR3_DOWN));
	scw.setUploadHour3Value(csm->get(SetupManager::SCHEDULER_HOUR3_UP));
	scw.setDownloadHour4Value(csm->get(SetupManager::SCHEDULER_HOUR4_DOWN));
	scw.setUploadHour4Value(csm->get(SetupManager::SCHEDULER_HOUR4_UP));
	scw.setDownloadHour5Value(csm->get(SetupManager::SCHEDULER_HOUR5_DOWN));
	scw.setUploadHour5Value(csm->get(SetupManager::SCHEDULER_HOUR5_UP));
	scw.setDownloadHour6Value(csm->get(SetupManager::SCHEDULER_HOUR6_DOWN));
	scw.setUploadHour6Value(csm->get(SetupManager::SCHEDULER_HOUR6_UP));
	scw.setDownloadHour7Value(csm->get(SetupManager::SCHEDULER_HOUR7_DOWN));
	scw.setUploadHour7Value(csm->get(SetupManager::SCHEDULER_HOUR7_UP));
	scw.setDownloadHour8Value(csm->get(SetupManager::SCHEDULER_HOUR8_DOWN));
	scw.setUploadHour8Value(csm->get(SetupManager::SCHEDULER_HOUR8_UP));
	scw.setDownloadHour9Value(csm->get(SetupManager::SCHEDULER_HOUR9_DOWN));
	scw.setUploadHour9Value(csm->get(SetupManager::SCHEDULER_HOUR9_UP));
	scw.setDownloadHour10Value(csm->get(SetupManager::SCHEDULER_HOUR10_DOWN));
	scw.setUploadHour10Value(csm->get(SetupManager::SCHEDULER_HOUR10_UP));
	scw.setDownloadHour11Value(csm->get(SetupManager::SCHEDULER_HOUR11_DOWN));
	scw.setUploadHour11Value(csm->get(SetupManager::SCHEDULER_HOUR11_UP));
	scw.setDownloadHour12Value(csm->get(SetupManager::SCHEDULER_HOUR12_DOWN));
	scw.setUploadHour12Value(csm->get(SetupManager::SCHEDULER_HOUR12_UP));
	scw.setDownloadHour13Value(csm->get(SetupManager::SCHEDULER_HOUR13_DOWN));
	scw.setUploadHour13Value(csm->get(SetupManager::SCHEDULER_HOUR13_UP));
	scw.setDownloadHour14Value(csm->get(SetupManager::SCHEDULER_HOUR14_DOWN));
	scw.setUploadHour14Value(csm->get(SetupManager::SCHEDULER_HOUR14_UP));
	scw.setDownloadHour15Value(csm->get(SetupManager::SCHEDULER_HOUR15_DOWN));
	scw.setUploadHour15Value(csm->get(SetupManager::SCHEDULER_HOUR15_UP));
	scw.setDownloadHour16Value(csm->get(SetupManager::SCHEDULER_HOUR16_DOWN));
	scw.setUploadHour16Value(csm->get(SetupManager::SCHEDULER_HOUR16_UP));
	scw.setDownloadHour17Value(csm->get(SetupManager::SCHEDULER_HOUR17_DOWN));
	scw.setUploadHour17Value(csm->get(SetupManager::SCHEDULER_HOUR17_UP));
	scw.setDownloadHour18Value(csm->get(SetupManager::SCHEDULER_HOUR18_DOWN));
	scw.setUploadHour18Value(csm->get(SetupManager::SCHEDULER_HOUR18_UP));
	scw.setDownloadHour19Value(csm->get(SetupManager::SCHEDULER_HOUR19_DOWN));
	scw.setUploadHour19Value(csm->get(SetupManager::SCHEDULER_HOUR19_UP));
	scw.setDownloadHour20Value(csm->get(SetupManager::SCHEDULER_HOUR20_DOWN));
	scw.setUploadHour20Value(csm->get(SetupManager::SCHEDULER_HOUR20_UP));
	scw.setDownloadHour21Value(csm->get(SetupManager::SCHEDULER_HOUR21_DOWN));
	scw.setUploadHour21Value(csm->get(SetupManager::SCHEDULER_HOUR21_UP));
	scw.setDownloadHour22Value(csm->get(SetupManager::SCHEDULER_HOUR22_DOWN));
	scw.setUploadHour22Value(csm->get(SetupManager::SCHEDULER_HOUR22_UP));
	scw.setDownloadHour23Value(csm->get(SetupManager::SCHEDULER_HOUR23_DOWN));
	scw.setUploadHour23Value(csm->get(SetupManager::SCHEDULER_HOUR23_UP));
}


// CONNECTION OPTIONS
void OptionsTab::setFirstTcpButtonConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::FIRST_PORT, cw.getFirstTcpButtonData());
	Lince::Core::get_session_manager()->update_ports();
}

void OptionsTab::setLastTcpButtonConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::LAST_PORT, cw.getLastTcpButtonData());
	Lince::Core::get_session_manager()->update_ports();
}

void OptionsTab::setDhtConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::DHT_SUPPORT, cw.getDhtActivateButtonData());
	Lince::Core::get_session_manager()->update_dht();
}

void OptionsTab::setUpnpConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::UPNP_SUPPORT, cw.getUpnpActivateButtonData());
	Lince::Core::get_session_manager()->update_upnp();
}

void OptionsTab::setLocalDiscoveryConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::LOCAL_DISCOVERY_SUPPORT, cw.getLocalDiscoveryActivateButtonData());
	Lince::Core::get_session_manager()->update_lsd();
}

void OptionsTab::setMetadataConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::METADATA_SUPPORT, cw.getMetadataActivateButtonData());
}

void OptionsTab::setPexConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::PEX_SUPPORT, cw.getPexActivateButton());
}

void OptionsTab::setSmartBanValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SMARTBAN_SUPPORT, cw.getSmartBanButton());
}

void OptionsTab::setAdvancedBindToInterfaceConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::BIND_TO_INTERFACE, cw.getAdvancedBindToInterfaceEntry());
}

void OptionsTab::setAdvancedMaxHalfOpenConnectionsConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::MAX_HALF_OPEN_CONNECTIONS, cw.getAdvancedMaxHalfOpenConnectionsButton());
}


// PROXY OPTIONS
void OptionsTab::setServerProxyNameConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SERVER_PROXY_NAME, pw.getServerProxyNameEntry());
	Lince::Core::get_session_manager()->update_proxy_settings();
}

void OptionsTab::setServerProxyPortConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SERVER_PROXY_PORT, pw.getServerProxyPortButton());
	Lince::Core::get_session_manager()->update_proxy_settings();
}

void OptionsTab::setServerProxyLoginConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SERVER_PROXY_USERNAME, pw.getServerProxyLoginEntry());
	Lince::Core::get_session_manager()->update_proxy_settings();
}

void OptionsTab::setServerProxyPassConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SERVER_PROXY_PASSWORD, pw.getServerProxyPassEntry());
	Lince::Core::get_session_manager()->update_proxy_settings();
}

void OptionsTab::setServerProxyEnableCheckButtonConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SERVER_USE_PROXY, pw.getServerProxyEnableCheckButton());
	Lince::Core::get_session_manager()->update_proxy_settings();
}

void OptionsTab::setServerProxyTypeConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SERVER_PROXY_TYPE, pw.getServerProxyType());
	Lince::Core::get_session_manager()->update_proxy_settings();
}

void OptionsTab::setClientProxyNameConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::CLIENT_PROXY_NAME, pw.getClientProxyNameEntry());
	Lince::Core::get_session_manager()->update_proxy_settings();
}

void OptionsTab::setClientProxyPortConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::CLIENT_PROXY_PORT, pw.getClientProxyPortButton());
	Lince::Core::get_session_manager()->update_proxy_settings();
}

void OptionsTab::setClientProxyLoginConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::CLIENT_PROXY_USERNAME, pw.getClientProxyLoginEntry());
	Lince::Core::get_session_manager()->update_proxy_settings();
}

void OptionsTab::setClientProxyPassConfigValue(){
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::CLIENT_PROXY_PASSWORD, pw.getClientProxyPassEntry());
	Lince::Core::get_session_manager()->update_proxy_settings();
}

void OptionsTab::setClientProxyEnableCheckButtonConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::CLIENT_USE_PROXY, pw.getClientProxyEnableCheckButton());
	Lince::Core::get_session_manager()->update_proxy_settings();
}

void OptionsTab::setClientProxyTypeConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::CLIENT_PROXY_TYPE, pw.getClientProxyType());
	Lince::Core::get_session_manager()->update_proxy_settings();
}

// ENCRYPTION WINDOWS
void OptionsTab::setEncryptionEnableButtonConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::USE_ENCRYPTION, ew.getEncryptionEnableCheckButton());
	Lince::Core::get_session_manager()->update_encryption_support();
}

void OptionsTab::setPreferRC4ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::ENCRYPTION_PREFER_RC4, ew.getPreferRC4());
	Lince::Core::get_session_manager()->update_encryption_support();
}

void OptionsTab::setInconmingConnectionsButtonConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::ENCRYPTED_INCOMING_CONNECTIONS, ew.getInconmingConnectionsButtonData());
	Lince::Core::get_session_manager()->update_encryption_support();
}

void OptionsTab::setOutgoingConnectionsButtonConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::ENCRYPTED_OUTGOING_CONNECTIONS, ew.getOutgoingConnectionsButtonData());
	Lince::Core::get_session_manager()->update_encryption_support();
}

void OptionsTab::setAllowedEncryptionButtonConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::ENCRYPTION_LEVEL, ew.getAllowedEncryptionButtonData());
	Lince::Core::get_session_manager()->update_encryption_support();
}

// TRANSFERENCE OPTIONS
void OptionsTab::setMaxUploadButtonConfigValue()
{
	Lince::SessionManagerPtr ssm = Lince::Core::get_session_manager();
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::UPLOAD_SPEED, tw.getMaxUploadButton());
	ssm->set_upload_speed_session(tw.getMaxUploadButton());
}

void OptionsTab::setMaxDownloadButtonConfigValue()
{
	Lince::SessionManagerPtr ssm = Lince::Core::get_session_manager();
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::DOWNLOAD_SPEED, tw.getMaxDownloadButton());
	ssm->set_download_speed_session(tw.getMaxDownloadButton());
}

void OptionsTab::setMaxConnectsButtonConfigValue()
{
	Lince::SessionManagerPtr ssm = Lince::Core::get_session_manager();
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::CONNECTIONS_NUMBER, tw.getMaxConnectsButton());
	ssm->set_max_connections(tw.getMaxConnectsButton());
}

void OptionsTab::setMaxUploadSlotsButtonConfigValue()
{
	Lince::SessionManagerPtr ssm = Lince::Core::get_session_manager();
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::UPLOAD_SLOTS, tw.getMaxUploadSlotsButton());
	ssm->set_max_uploads(tw.getMaxUploadSlotsButton());
}

void OptionsTab::setMaxUploadCheckButtonConfigValue()
{
	Lince::SessionManagerPtr ssm = Lince::Core::get_session_manager();
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();

	csm->set(SetupManager::DISABLE_UPLOAD_SPEED, tw.getMaxUploadCheckButton());
	if (!tw.getMaxUploadCheckButton())
		setMaxUploadButtonConfigValue();
	else
		ssm->set_upload_speed_session(-1);
}

void OptionsTab::setMaxDownloadCheckButtonConfigValue()
{
	Lince::SessionManagerPtr ssm = Lince::Core::get_session_manager();
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();

	csm->set(SetupManager::DISABLE_DOWNLOAD_SPEED, tw.getMaxDownloadCheckButton());
	if (!tw.getMaxDownloadCheckButton())
		setMaxDownloadButtonConfigValue();
	else
		ssm->set_download_speed_session(-1);
}

void OptionsTab::setMaxConnectsCheckButtonConfigValue()
{
	Lince::SessionManagerPtr ssm = Lince::Core::get_session_manager();
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();

	csm->set(SetupManager::DISABLE_CONNECTIONS_NUMBER, tw.getMaxConnectsCheckButton());
	if (!tw.getMaxConnectsCheckButton())
		setMaxConnectsButtonConfigValue();
	else
		ssm->set_max_connections(-1);
}

void OptionsTab::setMaxUploadSlotsCheckButtonConfigValue()
{
	Lince::SessionManagerPtr ssm = Lince::Core::get_session_manager();
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();

	csm->set(SetupManager::DISABLE_UPLOAD_SLOTS, tw.getMaxUploadSlotsCheckButton());
	if (!tw.getMaxUploadSlotsCheckButton())
		setMaxUploadSlotsButtonConfigValue();
	else
		ssm->set_max_uploads(-1);
}

void OptionsTab::setShareRatioConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SHARE_RATIO_1, tw.getShareRatioButton1());
	csm->set(SetupManager::SHARE_RATIO_2, tw.getShareRatioButton2());
}

void OptionsTab::setShareRatioCheckButtonConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::DISABLE_SHARE_RATIO, tw.getShareRatioCheckButton());
}

void OptionsTab::setAllowConnectionsFromTheSameIpConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::CONNECTIONS_FROM_THE_SAME_IP, tw.getAllowConnectionsFromTheSameIp());
}

void OptionsTab::setPrioritizeFirstPartOfFilesConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::PRIORITIZE_FIRST_PART_OF_FILES, tw.getPrioritizeFirstPartOfFiles());
}

void OptionsTab::setMaxConfigUploadSpeed(int value)
{
	if (value == -1)
		tw.setMaxUploadCheckButton(true);
	else{
		tw.setMaxUploadCheckButton(false);
		tw.setMaxUploadButton(value);
	}
}

void OptionsTab::setMaxConfigDownloadSpeed(int value)
{
	if (value == -1)
		tw.setMaxDownloadCheckButton(true);
	else{
		tw.setMaxDownloadCheckButton(false);
		tw.setMaxDownloadButton(value);
	}
}

void OptionsTab::setMaxConfigConnects(int value)
{
	if (value == -1)
		tw.setMaxConnectsCheckButton(true);
	else{
		tw.setMaxConnectsCheckButton(false);
		tw.setMaxConnectsButton(value);
	}
}

void OptionsTab::setMaxConfigUploadSlots(int value)
{
	if (value == -1)
		tw.setMaxUploadSlotsCheckButton(true);
	else{
		tw.setMaxUploadSlotsCheckButton(false);
		tw.setMaxUploadSlotsButton(value);
	}
}

// FILES OPTIONS
void OptionsTab::setDirectoryEntryConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();

	csm->set(SetupManager::DEFAULT_FILE_DIRECTORY, fw.getDirectoryEntry());
	gsm->set(GuiSetupManager::DEFAULT_FILE_DIRECTORY, fw.getDirectoryEntry());
}

void OptionsTab::setDownloadDefaultDirectoryCheckButtonConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();

	csm->set(SetupManager::USE_DEFAULT_FILE_DIRECTORY, fw.getDownloadDefaultDirectoryCheckButton());
	gsm->set(GuiSetupManager::USE_DEFAULT_FILE_DIRECTORY, fw.getDownloadDefaultDirectoryCheckButton());
}

void OptionsTab::setStorageTypeConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::STORAGE_TYPE, fw.getStorageType());
}

void OptionsTab::setFastResumeCheckButtonConfigValue()
{
	//gsm->set(GuiSetupManager::USE_FAST_RESUME, fw.getFastResumeCheckButton());
}

void OptionsTab::setFastResumeTimeUpdateButtonConfigValue()
{
	//gsm->set(GuiSetupManager::FAST_RESUME_TIME, fw.getFastResumeTimeUpdateButton());
}

void OptionsTab::setConfirmDataEliminationCheckButtonConfigValue()
{
	gsm->set(GuiSetupManager::CONFIRM_DATA_ELIMINATION, fw.getConfirmDataEliminationCheckButton());
}

void OptionsTab::setAskWhenLoadingTorrentsUsingDbusRequestsConfigValue()
{
	gsm->set(GuiSetupManager::ASK_WHEN_LOADING_FROM_DBUS_REQUEST,
			fw.getAskWhenLoadingTorrentsUsingDbusRequestsCheckButton());
}

// TORRENTS OPTIONS
void OptionsTab::setSaveTorrentsCheckButtonConfigValue()
{
	gsm->set(GuiSetupManager::USE_DOWNLOAD_TORRENTS_DIRECTORY, tow.getSaveTorrentsCheckButton());
}

void OptionsTab::setRemoveOriginalTorrentConfigValue()
{
	gsm->set(GuiSetupManager::USE_REMOVE_ORIGINAL_TORRENT, tow.getRemoveOriginalTorrent());
}

void OptionsTab::setRemoveTorrentWhenFinishedConfigValue()
{
	gsm->set(GuiSetupManager::USE_REMOVE_TORRENT_WHEN_FINISHED, tow.getRemoveTorrentWhenFinished());
}

void OptionsTab::setSaveDirectoryEntryConfigValue()
{
	gsm->set(GuiSetupManager::DOWNLOAD_TORRENTS_DIRECTORY, tow.getSaveDirectoryEntry());
}

void OptionsTab::setLoadTorrentsCheckButtonConfigValue()
{
	gsm->set(GuiSetupManager::USE_LOAD_TORRENTS_DIRECTORY, tow.getLoadTorrentsCheckButton());
}

void OptionsTab::setLoadDirectoryEntryConfigValue()
{
	gsm->set(GuiSetupManager::LOAD_TORRENTS_DIRECTORY, tow.getLoadDirectoryEntry());
}

void OptionsTab::setAllowChangeDirectoryConfigValue()
{
	gsm->set(GuiSetupManager::ALLOW_CHANGE_LOAD_DIRECTORY, tow.getAllowChangeDirectory());
}

// STATISTICS OPTIONS
void OptionsTab::setStatisticsTabCheckButtonConfigValue()
{
	gsm->set(GuiSetupManager::SHOW_GRAPHIC_STATISTICS, sw.getStatisticsTabCheckButton());
	nb->showGraphStats(sw.getStatisticsTabCheckButton());
}

void OptionsTab::setUpdateTimeButtonConfigValue()
{
	gsm->set(GuiSetupManager::STATISTICS_TIME_UPDATE, sw.getUpdateTimeButton());
}

// LOG OPTIONS
void OptionsTab::setActivateLogCheckButtonConfigValue()
{
	gsm->set(GuiSetupManager::USE_LOG_TAB, lw.getActivateLogCheckButton());
	nb->setLogTabActive(lw.getActivateLogCheckButton());
}

void OptionsTab::setColoredOutputCheckButtonConfigValue()
{
	gsm->set(GuiSetupManager::LOG_COLORED_OUTPUT, lw.getColoredOutputCheckButton());
	nb->setLogColoredOutput(lw.getColoredOutputCheckButton());
}

void OptionsTab::setSaveLogFileCheckButtonConfigValue()
{
	gsm->set(GuiSetupManager::SAVE_LOG_TO_FILE, lw.getSaveLogFileCheckButton());
	nb->setLogSaveToFile(lw.getSaveLogFileCheckButton());
}

void OptionsTab::setBufferSizeButtonConfigValue()
{
	gsm->set(GuiSetupManager::LOG_SIZE, lw.getBufferSizeButton());
	nb->setMaxLogSize(lw.getBufferSizeButton() * 1024);
}

void OptionsTab::setAutoScrollCheckButtonConfigValue()
{
	gsm->set(GuiSetupManager::LOG_AUTO_SCROLL, lw.getAutoScrollCheckButton());
	nb->setLogAutoScroll(lw.getAutoScrollCheckButton());
}

void OptionsTab::setLogLevelConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	gsm->set(GuiSetupManager::LOG_LEVEL, lw.getLogLevel());
	nb->setLogLevel(lw.getLogLevel());
}

// NOTIFICATIONS WINDOW
void OptionsTab::setActivateNotificationsConfigValue()
{
	gsm->set(GuiSetupManager::USE_NOTIFICATIONS, nw.getActivateNotificationsCheckButton());
	nb->setShowNotifications(gsm->getBool(GuiSetupManager::USE_NOTIFICATIONS));
}

void OptionsTab::setNotifyFinishedTorrentsConfigValue()
{
	gsm->set(GuiSetupManager::NOTIFY_FINISHED_TORRENTS, nw.getNotifyFinishedTorrentsCheckButton());
	nb->setNotifyMessage(Notifications::torrentFinished, nw.getNotifyFinishedTorrentsCheckButton());
}

void OptionsTab::setNotifyStoppedTorrentsConfigValue()
{
	gsm->set(GuiSetupManager::NOTIFY_STOPPED_TORRENTS, nw.getNotifyStoppedTorrentsCheckButton());
	nb->setNotifyMessage(Notifications::torrentStopped, nw.getNotifyStoppedTorrentsCheckButton());
}

void OptionsTab::setNotifyStartedTorrentsConfigValue()
{
	gsm->set(GuiSetupManager::NOTIFY_STARTED_TORRENTS, nw.getNotifyStartedTorrentsCheckButton());
	nb->setNotifyMessage(Notifications::torrentStarted, nw.getNotifyStartedTorrentsCheckButton());
}

void OptionsTab::setNotifyAddedTorrentsConfigValue()
{
	gsm->set(GuiSetupManager::NOTIFY_ADDED_TORRENTS, nw.getNotifyAddedTorrentsCheckButton());
	nb->setNotifyMessage(Notifications::torrentAdded, nw.getNotifyAddedTorrentsCheckButton());
}

void OptionsTab::setNotifyRemovedTorrentsConfigValue()
{
	gsm->set(GuiSetupManager::NOTIFY_REMOVED_TORRENTS, nw.getNotifyRemovedTorrentsCheckButton());
	nb->setNotifyMessage(Notifications::torrentRemoved, nw.getNotifyRemovedTorrentsCheckButton());
}

void OptionsTab::setNotifyQueuedTorrentsConfigValue()
{
	gsm->set(GuiSetupManager::NOTIFY_QUEUED_TORRENTS, nw.getNotifyQueuedTorrentsCheckButton());
	nb->setNotifyMessage(Notifications::torrentQueued, nw.getNotifyQueuedTorrentsCheckButton());
}

void OptionsTab::setNotifyUnqueuedTorrentsConfigValue()
{
	gsm->set(GuiSetupManager::NOTIFY_UNQUEUED_TORRENTS, nw.getNotifyUnqueuedTorrentsCheckButton());
	nb->setNotifyMessage(Notifications::torrentUnqueued, nw.getNotifyUnqueuedTorrentsCheckButton());
}

void OptionsTab::setNotifyShowFromSystrayConfigValue()
{
	gsm->set(GuiSetupManager::NOTIFY_SHOW_FROM_SYSTRAY, nw.getNotifyShowFromSystrayCheckButton());
	nb->setNotifyFromSystray(nw.getNotifyShowFromSystrayCheckButton());
}

// QUEUE WINDOW
void OptionsTab::setActiveDownloadsConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	Lince::TorrentsManagerPtr tm = Lince::Core::get_torrents_manager();
	csm->set(SetupManager::ACTIVE_DOWNLOADS_VALUE, qw.getActiveDownloadsQueueButton());
	tm->rebuild_downloading_torrents_queue();
}

void OptionsTab::setMaxDownloadsConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::MAX_DOWNLOADS_VALUE, qw.getMaxDownloadsQueueButton());
}

void OptionsTab::setActiveSharesConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	Lince::TorrentsManagerPtr tm = Lince::Core::get_torrents_manager();
	csm->set(SetupManager::ACTIVE_SHARES_VALUE, qw.getActiveSharedQueueButton());
	tm->rebuild_shared_torrents_queue();
}

// IP FILTER WINDOW
void OptionsTab::setActivateIpFilterConfigValue()
{
	Lince::SessionManagerPtr ssm = Lince::Core::get_session_manager();
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::USE_IP_FILTER, ifw.getActivateIpFilterCheckButton());
	ssm->update_ip_filter();
}

void OptionsTab::selectIpFilterFile()
{
	Lince::SessionManagerPtr ssm = Lince::Core::get_session_manager();
	Glib::ustring fileName;

	fileName = Utils::getFilenameFromLoadDialog(_("Select an ipfilter.dat file"),
												"", "Dat Files", "*.dat");

	if (fileName != "")
	{
		ssm->get_ip_filter()->load_from_file_v4(std::string(fileName.c_str()));
		ssm->update_ip_filter();
		ifw.updateIpFilterData();
	}
}

void OptionsTab::removeSelectedIpFilter()
{
	if (ifw.isRowSelected())
	{
		Glib::ustring range = ifw.getFirstSelectedIp() + "-" +
			ifw.getLastSelectedIp();
		Lince::SessionManagerPtr ssm = Lince::Core::get_session_manager();

		ssm->get_ip_filter()->add_ipv4_rule(range.c_str(), 128);
		ssm->update_ip_filter();
		ifw.updateIpFilterData();
	}
}

void OptionsTab::addIpFilter()
{
	Glib::ustring range;

	if (Utils::askUstringDialog(_("Enter an ip filter"),
								_("Enter an ip filter (FirstIp-LastIp)"), range))
	{
		if (range.empty()) return;

		Lince::SessionManagerPtr ssm = Lince::Core::get_session_manager();

		ssm->get_ip_filter()->add_ipv4_rule(range.c_str(),
											libtorrent::ip_filter::blocked);

		ssm->update_ip_filter();
		ifw.updateIpFilterData();
	}
}

void OptionsTab::banIP(const std::string &ip){
	std::string range = ip + " - " + ip;

	Lince::SessionManagerPtr ssm = Lince::Core::get_session_manager();

	ssm->get_ip_filter()->add_ipv4_rule(range.c_str(),
										libtorrent::ip_filter::blocked);

	ssm->update_ip_filter();
	ifw.updateIpFilterData();
}

// SEARCH WINDOW
void OptionsTab::setBtjunkieConfigValue()
{
	gsm->set(GuiSetupManager::USE_BTJUNKIE, shw.getBtjunkieCheckButton());
}

void OptionsTab::setIsohuntConfigValue()
{
	gsm->set(GuiSetupManager::USE_ISOHUNT, shw.getIsohuntCheckButton());
}

void OptionsTab::setPirateBayConfigValue()
{
	gsm->set(GuiSetupManager::USE_PIRATEBAY, shw.getPiratebayCheckButton());
}

void OptionsTab::setMininovaConfigValue()
{
	gsm->set(GuiSetupManager::USE_MININOVA, shw.getMininovaCheckButton());
}

void OptionsTab::setTorrentreactorConfigValue()
{
	gsm->set(GuiSetupManager::USE_TORRENTREACTOR, shw.getTorrentreactorCheckButton());
}

void OptionsTab::setFilterResultsWithoutSeedersConfigValue()
{
	gsm->set(GuiSetupManager::FILTER_RESULTS_WITHOUT_SEEDERS, shw.getFilterSeedersCheckButton());
}

void OptionsTab::setMaxSearchConfigValue()
{
	gsm->set(GuiSetupManager::MAX_NUMBER_OF_SEARCHES, shw.getMaxSearchesValue());
}

// SCHEDULER WINDOW
void OptionsTab::setUseSchedulerConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::USE_SCHEDULER, scw.getUseSchedulerCheckButton());
	updateSchedulerConfig();
}

void OptionsTab::updateSchedulerConfig()
{
	Lince::SessionManagerPtr ssm = Lince::Core::get_session_manager();
	ssm->update_upload_session_speed();
	ssm->update_download_session_speed();
}

void OptionsTab::setMondayConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_MONDAY, scw.getMondayCheckButton());
}

void OptionsTab::setTuesdayConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_TUESDAY, scw.getTuesdayCheckButon());
}

void OptionsTab::setWednesdayConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_WEDNESDAY, scw.getWednesdayCheckButton());
}

void OptionsTab::setThursdayConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_THURSDAY, scw.getThursdayCheckButton());
}

void OptionsTab::setFridayConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_FRIDAY, scw.getFridayCheckButton());
}

void OptionsTab::setSaturdayConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_SATURDAY, scw.getSaturdayCheckButton());
}

void OptionsTab::setSundayConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_SUNDAY, scw.getSundayCheckButton());
}

void OptionsTab::setDownloadHour0ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR0_DOWN, scw.getDownloadHour0Value());
}

void OptionsTab::setUploadHour0ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR0_UP, scw.getUploadHour0Value());
}

void OptionsTab::setDownloadHour1ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR1_DOWN, scw.getDownloadHour1Value());
}

void OptionsTab::setUploadHour1ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR1_UP, scw.getUploadHour1Value());
}

void OptionsTab::setDownloadHour2ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR2_DOWN, scw.getDownloadHour2Value());
}

void OptionsTab::setUploadHour2ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR2_UP, scw.getUploadHour2Value());
}

void OptionsTab::setDownloadHour3ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR3_DOWN, scw.getDownloadHour3Value());
}

void OptionsTab::setUploadHour3ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR3_UP, scw.getUploadHour3Value());
}

void OptionsTab::setDownloadHour4ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR4_DOWN, scw.getDownloadHour4Value());
}

void OptionsTab::setUploadHour4ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR4_UP, scw.getUploadHour4Value());
}

void OptionsTab::setDownloadHour5ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR5_DOWN, scw.getDownloadHour5Value());
}

void OptionsTab::setUploadHour5ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR5_UP, scw.getUploadHour5Value());
}

void OptionsTab::setDownloadHour6ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR6_DOWN, scw.getDownloadHour6Value());
}

void OptionsTab::setUploadHour6ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR6_UP, scw.getUploadHour6Value());
}

void OptionsTab::setDownloadHour7ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR7_DOWN, scw.getDownloadHour7Value());
}

void OptionsTab::setUploadHour7ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR7_UP, scw.getUploadHour7Value());
}

void OptionsTab::setDownloadHour8ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR8_DOWN, scw.getDownloadHour8Value());
}

void OptionsTab::setUploadHour8ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR8_UP, scw.getUploadHour8Value());
}

void OptionsTab::setDownloadHour9ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR9_DOWN, scw.getDownloadHour9Value());
}

void OptionsTab::setUploadHour9ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR9_UP, scw.getUploadHour9Value());
}

void OptionsTab::setDownloadHour10ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR10_DOWN, scw.getDownloadHour10Value());
}

void OptionsTab::setUploadHour10ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR10_UP, scw.getUploadHour10Value());
}

void OptionsTab::setDownloadHour11ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR11_DOWN, scw.getDownloadHour11Value());
}

void OptionsTab::setUploadHour11ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR11_UP, scw.getUploadHour11Value());
}

void OptionsTab::setDownloadHour12ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR12_DOWN, scw.getDownloadHour12Value());
}

void OptionsTab::setUploadHour12ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR12_UP, scw.getUploadHour12Value());
}

void OptionsTab::setDownloadHour13ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR13_DOWN, scw.getDownloadHour13Value());
}

void OptionsTab::setUploadHour13ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR13_UP, scw.getUploadHour13Value());
}

void OptionsTab::setDownloadHour14ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR14_DOWN, scw.getDownloadHour14Value());
}

void OptionsTab::setUploadHour14ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR14_UP, scw.getUploadHour14Value());
}

void OptionsTab::setDownloadHour15ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR15_DOWN, scw.getDownloadHour15Value());
}

void OptionsTab::setUploadHour15ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR15_UP, scw.getUploadHour15Value());
}

void OptionsTab::setDownloadHour16ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR16_DOWN, scw.getDownloadHour16Value());
}

void OptionsTab::setUploadHour16ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR16_UP, scw.getUploadHour16Value());
}

void OptionsTab::setDownloadHour17ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR17_DOWN, scw.getDownloadHour17Value());
}

void OptionsTab::setUploadHour17ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR17_UP, scw.getUploadHour17Value());
}

void OptionsTab::setDownloadHour18ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR18_DOWN, scw.getDownloadHour18Value());
}

void OptionsTab::setUploadHour18ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR18_UP, scw.getUploadHour18Value());
}

void OptionsTab::setDownloadHour19ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR19_DOWN, scw.getDownloadHour19Value());
}

void OptionsTab::setUploadHour19ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR19_UP, scw.getUploadHour19Value());
}

void OptionsTab::setDownloadHour20ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR20_DOWN, scw.getDownloadHour20Value());
}

void OptionsTab::setUploadHour20ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR20_UP, scw.getUploadHour20Value());
}

void OptionsTab::setDownloadHour21ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR21_DOWN, scw.getDownloadHour21Value());
}

void OptionsTab::setUploadHour21ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR21_UP, scw.getUploadHour21Value());
}

void OptionsTab::setDownloadHour22ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR22_DOWN, scw.getDownloadHour22Value());
}

void OptionsTab::setUploadHour22ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR22_UP, scw.getUploadHour22Value());
}

void OptionsTab::setDownloadHour23ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR23_DOWN, scw.getDownloadHour23Value());
}

void OptionsTab::setUploadHour23ConfigValue()
{
	Lince::SetupManagerPtr csm = Lince::Core::get_setup_manager();
	csm->set(SetupManager::SCHEDULER_HOUR23_UP, scw.getUploadHour23Value());
}
