/***************************************************************************
 *            OptionsWindows.h
 *
 *  Mon Apr  9 13:57:21 2007
 *  Copyright  2007  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#ifndef _OPTIONSWINDOWS_H
#define _OPTIONSWINDOWS_H

#include "Utils.h"

#include <gtkmm.h>

class OptionsTab;

class OptionsWindowsBase
{
	private:
		Gtk::Label headerLabelText;

	protected:
		Gtk::HBox headerText;

	public:
		OptionsWindowsBase(Glib::ustring);
		OptionsWindowsBase();
		~OptionsWindowsBase();
};

class ConnectionWindow : public OptionsWindowsBase, public Gtk::ScrolledWindow
{
	private:
		// Global vbox
		Gtk::VBox vbox;

		// Tcp Row
		Gtk::Label tcpText1, tcpText2;
		Gtk::SpinButton firstTcpButton, lastTcpButton;
		Gtk::HBox tcpRow;

		// Options VBox
		Gtk::VBox optionsVBox;

		// DHT Row
		Gtk::CheckButton dhtActivateButton;
		Gtk::HBox dhtFrameHBox;

		// UPNP Row
		Gtk::CheckButton upnpActivateButton;
		Gtk::HBox upnpFrameHBox;

		// Local Discovery Row
		Gtk::CheckButton localDiscoveryActivateButton;
		Gtk::HBox localDiscoveryFrameHBox;

		// Metadata Row
		Gtk::CheckButton metadataActivateButton;
		Gtk::HBox metadataFrameHBox;

		// PeX support
		Gtk::CheckButton pexActivateButton;
		Gtk::HBox pexFrameHBox;

		// Smart Ban support
		Gtk::CheckButton smartBanButton;
		Gtk::HBox sbFrameHBox;

		// Advanced_options Row
		Gtk::Frame advancedFrame;
		Gtk::HBox advancedRow, advancedFrameHBox;
		Gtk::VBox advancedFrameVBox;
		Gtk::HBox advancedFrameHBox1, advancedFrameHBox2;
		Gtk::Label advancedBindToInterfaceLabel, advancedMaxHalfOpenConnectionsLabel;
		Gtk::Entry advancedBindToInterfaceEntry;
		Gtk::SpinButton advancedMaxHalfOpenConnectionsButton;

		// Signals
		void firstTcpButtonChanged();
		void lastTcpButtonChanged();

	public:
		ConnectionWindow(OptionsTab *);
		~ConnectionWindow();

		// Get options
		int getFirstTcpButtonData() { return firstTcpButton.get_value_as_int(); };
		int getLastTcpButtonData() { return lastTcpButton.get_value_as_int(); };
		bool getDhtActivateButtonData() { return dhtActivateButton.get_active(); };
		bool getUpnpActivateButtonData() { return upnpActivateButton.get_active(); };
		bool getLocalDiscoveryActivateButtonData() { return localDiscoveryActivateButton.get_active(); };
		bool getMetadataActivateButtonData() { return metadataActivateButton.get_active(); };
		bool getPexActivateButton() { return pexActivateButton.get_active(); };
		bool getSmartBanButton() { return smartBanButton.get_active(); };
		Glib::ustring getAdvancedBindToInterfaceEntry() { return advancedBindToInterfaceEntry.get_text(); };
		int getAdvancedMaxHalfOpenConnectionsButton() { return advancedMaxHalfOpenConnectionsButton.get_value_as_int(); };

		// Set options
		void setFirstTcpButtonData(int value) { firstTcpButton.set_value((double) value); };
		void setLastTcpButtonData(int value) { lastTcpButton.set_value((double) value); };
		void setDhtActivateButtonData(bool value) { dhtActivateButton.set_active(value); };
		void setUpnpActivateButtonData(bool value) { upnpActivateButton.set_active(value); };
		void setLocalDiscoveryActivateButtonData(bool value) { localDiscoveryActivateButton.set_active(value); };
		void setMetadataActivateButtonData(bool value) { metadataActivateButton.set_active(value); };
		void setPexActivateButton(bool value) { pexActivateButton.set_active(value); };
		void setSmartBanButton(bool value) { smartBanButton.set_active(value); };
		void setAdvancedBindToInterfaceEntry(const Glib::ustring & value) { advancedBindToInterfaceEntry.set_text(value); };
		void setAdvancedMaxHalfOpenConnectionsButton(int value) { advancedMaxHalfOpenConnectionsButton.set_value((double) value); };
};

class ProxyWindow : public OptionsWindowsBase, public Gtk::ScrolledWindow
{
	private:
		// Global vbox
		Gtk::VBox vbox;

		Utils::ModelColumnsIntString m_Columns;
		Glib::RefPtr<Gtk::ListStore> m_refTreeModel;

		// Proxy_Options Row
		Gtk::Frame serverProxyFrame, serverProxyFrame2;
		Gtk::HBox serverProxyRow, serverProxyFrameHBox;
		Gtk::VBox serverProxyFrameVBox, serverProxyFrameVBox2;
		Gtk::HBox serverProxyFrameHBox1, serverProxyFrameHBox2, serverProxyFrameHBox3, serverProxyFrameHBox4;
		Gtk::Label serverProxyNameLabel, serverProxyPortLabel, serverProxyLoginLabel, serverProxyPassLabel;
		Gtk::Entry serverProxyNameEntry, serverProxyLoginEntry, serverProxyPassEntry;
		Gtk::SpinButton serverProxyPortButton;
		Gtk::CheckButton serverProxyEnableCheckButton;

		Gtk::ComboBox serverProxyTypeCombo;
		Gtk::Label serverProxyTypeLabel;
		Gtk::HBox serverProxyTypeRow;

		// Client proxy
		Gtk::Frame clientProxyFrame, clientProxyFrame2;
		Gtk::HBox clientProxyRow, clientProxyFrameHBox;
		Gtk::VBox clientProxyFrameVBox, clientProxyFrameVBox2;
		Gtk::HBox clientProxyFrameHBox1, clientProxyFrameHBox2, clientProxyFrameHBox3, clientProxyFrameHBox4;
		Gtk::Label clientProxyNameLabel, clientProxyPortLabel, clientProxyLoginLabel, clientProxyPassLabel;
		Gtk::Entry clientProxyNameEntry, clientProxyLoginEntry, clientProxyPassEntry;
		Gtk::SpinButton clientProxyPortButton;
		Gtk::CheckButton clientProxyEnableCheckButton;

		Gtk::ComboBox clientProxyTypeCombo;
		Gtk::Label clientProxyTypeLabel;
		Gtk::HBox clientProxyTypeRow;

		// Signals
		void setProxyEnableSeverSensitivity();
		void setProxyEnableClientSensitivity();

	public:
		ProxyWindow(OptionsTab *);
		~ProxyWindow();

		// Get options
		Glib::ustring getServerProxyNameEntry() { return serverProxyNameEntry.get_text(); };
		int getServerProxyPortButton()  { return serverProxyPortButton.get_value_as_int(); };
		Glib::ustring getServerProxyLoginEntry() { return serverProxyLoginEntry.get_text(); };
		Glib::ustring getServerProxyPassEntry()  { return serverProxyPassEntry.get_text(); };
		bool getServerProxyEnableCheckButton() { return serverProxyEnableCheckButton.get_active(); };
		int getServerProxyType() { return serverProxyTypeCombo.get_active()->get_value(m_Columns.m_col_id); };

		Glib::ustring getClientProxyNameEntry() { return clientProxyNameEntry.get_text(); };
		int getClientProxyPortButton()  { return clientProxyPortButton.get_value_as_int(); };
		Glib::ustring getClientProxyLoginEntry() { return clientProxyLoginEntry.get_text(); };
		Glib::ustring getClientProxyPassEntry()  { return clientProxyPassEntry.get_text(); };
		bool getClientProxyEnableCheckButton() { return clientProxyEnableCheckButton.get_active(); };
		int getClientProxyType() { return clientProxyTypeCombo.get_active()->get_value(m_Columns.m_col_id); };

		// Set options
		void setServerProxyNameEntry(const Glib::ustring & value) { serverProxyNameEntry.set_text(value); };
		void setServerProxyPortButton(int value)  { serverProxyPortButton.set_value(value); };
		void setServerProxyLoginEntry(const Glib::ustring & value) { serverProxyLoginEntry.set_text(value); };
		void setServerProxyPassEntry(const Glib::ustring & value)  { serverProxyPassEntry.set_text(value); };
		void setServerProxyEnableCheckButton(bool value) { serverProxyEnableCheckButton.set_active(value); };
		void setServerProxyType(int value) { serverProxyTypeCombo.set_active(value-1); };

		void setClientProxyNameEntry(const Glib::ustring & value) { clientProxyNameEntry.set_text(value); };
		void setClientProxyPortButton(int value)  { clientProxyPortButton.set_value(value); };
		void setClientProxyLoginEntry(const Glib::ustring & value) { clientProxyLoginEntry.set_text(value); };
		void setClientProxyPassEntry(const Glib::ustring & value)  { clientProxyPassEntry.set_text(value); };
		void setClientProxyEnableCheckButton(bool value) { clientProxyEnableCheckButton.set_active(value); };
		void setClientProxyType(int value) { clientProxyTypeCombo.set_active(value-1); };
};

class EncryptionWindow : public OptionsWindowsBase, public Gtk::ScrolledWindow
{
	private:
		// Global vbox
		Gtk::VBox vbox;

		// Enable Encryption
		Gtk::CheckButton encryptionEnableCheckButton;
		Gtk::HBox encryptionEnableRow;

		// Incoming Frame
		Gtk::Frame incomingFrame;
		Gtk::HBox incomingFrameHBox, incomingFrameRow;
		Gtk::VBox incomingFrameVBox;
		Gtk::RadioButton incomingRadioButton1, incomingRadioButton2, incomingRadioButton3;

		// Outgoing Frame
		Gtk::Frame outgoingFrame;
		Gtk::HBox outgoingFrameHBox, outgoingFrameRow;
		Gtk::VBox outgoingFrameVBox;
		Gtk::RadioButton outgoingRadioButton1, outgoingRadioButton2, outgoingRadioButton3;

		// Allowed Encryption
		Gtk::Frame allowedEncryption;
		Gtk::HBox allowedEncryptionHBox, allowedEncryptionRow;
		Gtk::VBox allowedEncryptionVBox;
		Gtk::RadioButton allowedEncryptionButton1, allowedEncryptionButton2, allowedEncryptionButton3;

		// Enable Encryption
		Gtk::CheckButton preferRC4;
		Gtk::HBox preferRC4Row;

		void setEncryptionSensitivity();

	public:
		EncryptionWindow(OptionsTab *);
		~EncryptionWindow();

		// Get options
		bool getEncryptionEnableCheckButton() { return encryptionEnableCheckButton.get_active(); };
		bool getPreferRC4() { return preferRC4.get_active(); };
		int getInconmingConnectionsButtonData();
		int getOutgoingConnectionsButtonData();
		int getAllowedEncryptionButtonData();

		// Set options
		void setEncryptionEnableCheckButton(bool value) { encryptionEnableCheckButton.set_active(value); };
		void setPreferRC4(bool value) { preferRC4.set_active(value); };
		void setInconmingConnectionsButtonData(int);
		void setOutgoingConnectionsButtonData(int);
		void setAllowedEncryptionButtonData(int);
};

class TransferenceWindow : public OptionsWindowsBase, public Gtk::ScrolledWindow
{
	private:
		// Global vbox
		Gtk::VBox vbox;

		// Max Upload/Download/Connects/Upload slots
		Gtk::Label maxUploadLabel, maxDownloadLabel, maxConnectsLabel, maxUploadSlotsLabel;
		Gtk::SpinButton maxUploadButton, maxDownloadButton, maxConnectsButton, maxUploadSlotsButton;
		Gtk::CheckButton maxUploadCheckButton, maxDownloadCheckButton, maxConnectsCheckButton, maxUploadSlotsCheckButton;

		// Share Ratio
		Gtk::Label shareRatioLabel1, shareRatioLabel2, shareRatioLabel3;
		Gtk::SpinButton shareRatioButton;
		Gtk::CheckButton shareRatioCheckButton;

		// Connections from the same ip
		Gtk::CheckButton allowConnectionsFromTheSameIp;

		// Prioritize the first part of the files.
		Gtk::CheckButton prioritizeFirstPartOfFiles;

		// Signals
		void setMaxUploadButtonSensitivity();
		void setMaxDownloadButtonSensitivity();
		void setMaxConnectsButtonSensitivity();
		void setMaxUploadSlotsButtonSensitivity();
		void setShareRatioButtonSensitivity();

	public:
		TransferenceWindow(OptionsTab *);
		~TransferenceWindow();

		// Get options
		int getMaxUploadButton()   { return maxUploadButton.get_value_as_int(); };
		int getMaxDownloadButton() { return maxDownloadButton.get_value_as_int(); };
		int getMaxConnectsButton() { return maxConnectsButton.get_value_as_int(); };
		int getMaxUploadSlotsButton() { return maxUploadSlotsButton.get_value_as_int(); };
		bool getMaxUploadCheckButton() { return maxUploadCheckButton.get_active(); };
		bool getMaxDownloadCheckButton() { return maxDownloadCheckButton.get_active(); };
		bool getMaxConnectsCheckButton() { return maxConnectsCheckButton.get_active(); };
		bool getMaxUploadSlotsCheckButton() { return maxUploadSlotsCheckButton.get_active(); };
		int getShareRatioButton1(); int getShareRatioButton2();
		bool getShareRatioCheckButton() { return shareRatioCheckButton.get_active(); };
		bool getAllowConnectionsFromTheSameIp() { return allowConnectionsFromTheSameIp.get_active(); };
		bool getPrioritizeFirstPartOfFiles() { return prioritizeFirstPartOfFiles.get_active(); };

		// Set options
		void setMaxUploadButton(int value)   { maxUploadButton.set_value((double) value); };
		void setMaxDownloadButton(int value) { maxDownloadButton.set_value((double) value); };
		void setMaxConnectsButton(int value) { maxConnectsButton.set_value((double) value); };
		void setMaxUploadSlotsButton(int value) { maxUploadSlotsButton.set_value((double) value); };
		void setMaxUploadCheckButton(bool value) { maxUploadCheckButton.set_active(value);};
		void setMaxDownloadCheckButton(bool value) { maxDownloadCheckButton.set_active(value); };
		void setMaxConnectsCheckButton(bool value) { maxConnectsCheckButton.set_active(value); };
		void setMaxUploadSlotsCheckButton(bool value) { maxUploadSlotsCheckButton.set_active(value); };
		void setShareRatioButton(int value1, int value2) { shareRatioButton.set_value((double) value1 + (double) value2 / 10.0); };
		void setShareRatioCheckButton(bool value) { shareRatioCheckButton.set_active(value); };
		void setAllowConnectionsFromTheSameIp(bool value) { allowConnectionsFromTheSameIp.set_active(value); };
		void setPrioritizeFirstPartOfFiles(bool value) { prioritizeFirstPartOfFiles.set_active(value); };
};

class FilesWindow : public OptionsWindowsBase, public Gtk::ScrolledWindow
{
	private:
		// Global vbox
		Gtk::VBox vbox;

		// Default directory
		Gtk::Frame directoryFrame;
		Gtk::Entry directoryEntry;
		Gtk::Label directoryLabel;
		Gtk::CheckButton downloadDefaultDirectoryCheckButton;
		Gtk::Image directorySelectButtonImage;
		Gtk::Button directorySelectButton;
		Gtk::HBox directoryFrameHBox, directoryFrameHBox2, directoryRow;
		Gtk::VBox directoryFrameVBox;

		// Storage allocation
		Gtk::ComboBox storageTypeCombo;
		Gtk::Label storageTypeLabel;
		Gtk::HBox storageTypeRow;
		Utils::ModelColumnsIntString m_Columns;
		Glib::RefPtr<Gtk::ListStore> m_refTreeModel;

		// TODO: Fast resume
		// Fast resume is disabled by now util the option will be implemented by
		// Liblince
		// Fast resume
		// Gtk::CheckButton fastResumeCheckButton;
		// Gtk::HBox fastResumeRow, fastResumeTimeUpdateRow;
		// Gtk::Label fastResumeTimeUpdateLabel, minutesLabel;
		// Gtk::SpinButton fastResumeTimeUpdateButton;

		// Data elimination
		Gtk::CheckButton confirmDataEliminationCheckButton;
		Gtk::HBox confirmDataEliminationRow;

		// Ask when loading torrents from dbus requests
		Gtk::CheckButton askWhenLoadingTorrentsUsingDbusRequestsCheckButton;
		Gtk::HBox askWhenLoadingTorrentsUsingDbusRequestsRow;

		// Signals
		void selectAndSetDirectory();
		void setFastResumeTimeUpdateButtonSensitivity();

	public:
		FilesWindow(OptionsTab *);
		~FilesWindow();

		// Get options
		Glib::ustring getDirectoryEntry() { return directoryEntry.get_text(); };
		bool getDownloadDefaultDirectoryCheckButton() { return downloadDefaultDirectoryCheckButton.get_active(); };
		int getStorageType() { return storageTypeCombo.get_active()->get_value(m_Columns.m_col_id); };
		//bool getFastResumeCheckButton() { return fastResumeCheckButton.get_active(); };
		//int getFastResumeTimeUpdateButton() { return fastResumeTimeUpdateButton.get_value_as_int(); };
		bool getConfirmDataEliminationCheckButton() { return confirmDataEliminationCheckButton.get_active(); };
		bool getAskWhenLoadingTorrentsUsingDbusRequestsCheckButton() {
			return askWhenLoadingTorrentsUsingDbusRequestsCheckButton.get_active();
		};

		// Set options
		void setDirectoryEntry(Glib::ustring value) { directoryEntry.set_text(value); };
		void setDownloadDefaultDirectoryCheckButton(bool value) { downloadDefaultDirectoryCheckButton.set_active(value); };
		void setStorageType(int value) { storageTypeCombo.set_active(value-1); };
		//void setFastResumeCheckButton(bool value) { fastResumeCheckButton.set_active(value); };
		//void setFastResumeTimeUpdateButton(int value) { fastResumeTimeUpdateButton.set_value((double) value); };
		void setConfirmDataEliminationCheckButton(bool value) { confirmDataEliminationCheckButton.set_active(value); };
		void setAskWhenLoadingTorrentsUsingDbusRequestsCheckButton(bool value) {
			askWhenLoadingTorrentsUsingDbusRequestsCheckButton.set_active(value);
		};

};

class TorrentsOptionsWindow: public OptionsWindowsBase, public Gtk::ScrolledWindow
{
	private:
		// Global vbox
		Gtk::VBox vbox;

		// Save torrents file
		Gtk::CheckButton saveTorrentsCheckButton;
		Gtk::Label saveDirectoryLabel;
		Gtk::Entry saveDirectoryEntry;
		Gtk::Image saveDirectorySelectButtonImage;
		Gtk::Button saveDirectorySelectButton;
		Gtk::CheckButton removeOriginalTorrent, removeTorrentWhenFinished;

		// Load torrents file
		Gtk::CheckButton loadTorrentsCheckButton;
		Gtk::Label loadDirectoryLabel;
		Gtk::Entry loadDirectoryEntry;
		Gtk::Image loadDirectorySelectButtonImage;
		Gtk::Button loadDirectorySelectButton;
		Gtk::CheckButton allowChangeDirectory;

		// Signals
		void selectAndSetSaveDirectory();
		void setSaveDirectoryRowSensitivity();

		void selectAndSetLoadDirectory();
		void setLoadDirectoryRowSensitivity();

	public:
		TorrentsOptionsWindow(OptionsTab *);
		~TorrentsOptionsWindow();

		// Get options
		bool getSaveTorrentsCheckButton() { return saveTorrentsCheckButton.get_active(); };
		Glib::ustring getSaveDirectoryEntry() { return saveDirectoryEntry.get_text(); };
		bool getRemoveOriginalTorrent() { return removeOriginalTorrent.get_active(); };
		bool getRemoveTorrentWhenFinished() { return removeTorrentWhenFinished.get_active(); };

		bool getLoadTorrentsCheckButton() { return loadTorrentsCheckButton.get_active(); };
		Glib::ustring getLoadDirectoryEntry() { return loadDirectoryEntry.get_text(); };
		bool getAllowChangeDirectory() { return allowChangeDirectory.get_active(); };

		// Set options
		void setSaveTorrentsCheckButton(bool value) { saveTorrentsCheckButton.set_active(value); };
		void setSaveDirectoryEntry(Glib::ustring value) { saveDirectoryEntry.set_text(value); };
		void setRemoveOriginalTorrent(bool value) { removeOriginalTorrent.set_active(value); };
		void setRemoveTorrentWhenFinished(bool value) { removeTorrentWhenFinished.set_active(value); };

		void setLoadTorrentsCheckButton(bool value) { loadTorrentsCheckButton.set_active(value); };
		void setLoadDirectoryEntry(Glib::ustring value) { loadDirectoryEntry.set_text(value); };
		void setAllowChangeDirectory(bool value) { allowChangeDirectory.set_active(value); };
};

class StatisticsWindow: public OptionsWindowsBase, public Gtk::ScrolledWindow
{
	private:
		// Global vbox
		Gtk::VBox vbox;

		// Activate statistics Tab
		Gtk::Frame statisticsTabFrame;
		Gtk::VBox statisticsTabVBox;
		Gtk::CheckButton useStatisticsTabCheckButton;
		Gtk::HBox updateTimeRow, statisticsTabRow, useStatisticsTabRow;
		Gtk::SpinButton updateTimeButton;
		Gtk::Label updateTimeLabel, secondsLabel;

		// Signals
		void setStatisticsTabSensivity();

	public:
		StatisticsWindow(OptionsTab *);
		~StatisticsWindow();

		// Get options
		bool getStatisticsTabCheckButton() { return useStatisticsTabCheckButton.get_active(); };
		int getUpdateTimeButton() { return updateTimeButton.get_value_as_int(); };

		// Set options
		void setStatisticsTabCheckButton(bool value) { useStatisticsTabCheckButton.set_active(value); };
		void setUpdateTimeButton(int value) { updateTimeButton.set_value((double) value); };
};

class LogWindow: public OptionsWindowsBase, public Gtk::ScrolledWindow
{
	private:
		// Global vbox
		Gtk::VBox vbox;

		// Activate log tab
		Gtk::CheckButton activateLogCheckButton;

		// Colored output
		Gtk::CheckButton coloredOutputCheckButton;
		Gtk::Label coloredOutputLabel;

		// Buffer size
		Gtk::SpinButton bufferSizeButton;
		Gtk::Label bufferSizeLabel, kB;

		// Save File
		Gtk::Label saveLogFileLabel;
		Gtk::CheckButton saveLogFileCheckButton;

		// Use auto-scroll
		Gtk::Label autoScrollLabel;
		Gtk::CheckButton autoScrollCheckButton;

		// Log level
		Gtk::ComboBox logLevelCombo;
		Gtk::Label logLevelLabel;
		Utils::ModelColumnsIntString m_Columns;
		Glib::RefPtr<Gtk::ListStore> m_refTreeModel;

		// Signals
		void setTabFrameSensivity();

	public:
		LogWindow(OptionsTab *);
		~LogWindow();

		// Get options
		bool getActivateLogCheckButton() { return activateLogCheckButton.get_active(); };
		bool getColoredOutputCheckButton() { return coloredOutputCheckButton.get_active(); };
		bool getSaveLogFileCheckButton() { return saveLogFileCheckButton.get_active(); };
		bool getAutoScrollCheckButton() { return autoScrollCheckButton.get_active(); };
		int getBufferSizeButton() { return bufferSizeButton.get_value_as_int(); };
		int getLogLevel() { return logLevelCombo.get_active()->get_value(m_Columns.m_col_id); };

		// Set options
		void setActivateLogCheckButton(bool value) { activateLogCheckButton.set_active(value); };
		void setColoredOutputCheckButton(bool value) { coloredOutputCheckButton.set_active(value); };
		void setSaveLogFileCheckButton(bool value) { saveLogFileCheckButton.set_active(value); };
		void setAutoScrollCheckButton(bool value) { autoScrollCheckButton.set_active(value); };
		void setBufferSizeButton(int value) { bufferSizeButton.set_value((double) value); };
		void setLogLevel(int value) { logLevelCombo.set_active(value-1); };


};

class NotificationsWindow: public OptionsWindowsBase, public Gtk::ScrolledWindow
{
	private:
		// Global vbox
		Gtk::VBox vbox;

		Gtk::Frame notificationsFrame1;
		Gtk::CheckButton activateNotificationsCheckButton, notifyFinishedTorrentsCheckButton;
		Gtk::CheckButton notifyStoppedTorrentsCheckButton, notifyStartedTorrentsCheckButton;
		Gtk::CheckButton notifyQueuedTorrentsCheckButton, notifyUnqueuedTorrentsCheckButton;
		Gtk::CheckButton notifyRemovedTorrentsCheckButton, notifyAddedTorrentsCheckButton;
		Gtk::CheckButton notifyShowFromSystrayCheckButton;

		// Signals
		void setNotificationsFrameSensivity();

	public:
		NotificationsWindow(OptionsTab *);
		~NotificationsWindow();

		// Get options
		bool getActivateNotificationsCheckButton() { return activateNotificationsCheckButton.get_active(); };
		bool getNotifyFinishedTorrentsCheckButton() { return notifyFinishedTorrentsCheckButton.get_active(); };
		bool getNotifyStoppedTorrentsCheckButton() { return notifyStoppedTorrentsCheckButton.get_active(); };
		bool getNotifyStartedTorrentsCheckButton() { return notifyStartedTorrentsCheckButton.get_active(); };
		bool getNotifyQueuedTorrentsCheckButton() { return notifyQueuedTorrentsCheckButton.get_active(); };
		bool getNotifyUnqueuedTorrentsCheckButton() { return notifyUnqueuedTorrentsCheckButton.get_active(); };
		bool getNotifyAddedTorrentsCheckButton() { return notifyAddedTorrentsCheckButton.get_active(); };
		bool getNotifyRemovedTorrentsCheckButton() { return notifyRemovedTorrentsCheckButton.get_active(); };
		bool getNotifyShowFromSystrayCheckButton() { return notifyShowFromSystrayCheckButton.get_active(); };

		// Set options
		void setActivateNotificationsCheckButton(bool value) { activateNotificationsCheckButton.set_active(value); };
		void setNotifyFinishedTorrentsCheckButton(bool value) { notifyFinishedTorrentsCheckButton.set_active(value); };
		void setNotifyStoppedTorrentsCheckButton(bool value) { notifyStoppedTorrentsCheckButton.set_active(value); };
		void setNotifyStartedTorrentsCheckButton(bool value) { notifyStartedTorrentsCheckButton.set_active(value); };
		void setNotifyQueuedTorrentsCheckButton(bool value) { notifyQueuedTorrentsCheckButton.set_active(value); };
		void setNotifyUnqueuedTorrentsCheckButton(bool value) { notifyUnqueuedTorrentsCheckButton.set_active(value); };
		void setNotifyAddedTorrentsCheckButton(bool value) { notifyAddedTorrentsCheckButton.set_active(value); };
		void setNotifyRemovedTorrentsCheckButton(bool value) { notifyRemovedTorrentsCheckButton.set_active(value); };
		void setNotifyShowFromSystrayCheckButton(bool value) { notifyShowFromSystrayCheckButton.set_active(value); };
};

class QueueWindow: public OptionsWindowsBase, public Gtk::ScrolledWindow
{
	private:
		// Global vbox
		Gtk::VBox vbox;

		// Downloads Queue
		Gtk::Label activeDownloadsQueueLabel, maxDownloadsQueueLabel;
		Gtk::SpinButton activeDownloadsQueueButton, maxDownloadsQueueButton;

		// Shared Queue
		Gtk::Label activeSharedQueueLabel;
		Gtk::SpinButton activeSharedQueueButton;

		// Signals
		void activeDownloadsQueueButtonChanged();
		void maxDownloadsQueueButtonChanged();

	public:
		QueueWindow(OptionsTab *);
		~QueueWindow();

		// Get options
		int getActiveDownloadsQueueButton() { return activeDownloadsQueueButton.get_value_as_int(); };
		int getMaxDownloadsQueueButton() { return maxDownloadsQueueButton.get_value_as_int(); };
		int getActiveSharedQueueButton() { return activeSharedQueueButton.get_value_as_int(); };

		// Set options
		void setActiveDownloadsQueueButton(int value) { activeDownloadsQueueButton.set_value((double) value); };
		void setMaxDownloadsQueueButton(int value) { maxDownloadsQueueButton.set_value((double) value); };
		void setActiveSharedQueueButton(int value) { activeSharedQueueButton.set_value((double) value); };

};

class IpFilterWindow: public OptionsWindowsBase, public Gtk::ScrolledWindow
{
	private:
		// Global vbox
		Gtk::VBox vbox;

		// Activate Ip Filter tab
		Gtk::CheckButton activateIpFilterCheckButton;
		Gtk::HBox activateIpFilterBox;

		// Load Ip Filter File
		Gtk::Label ipFilterFileLabel;
		Gtk::Image ipFilterFileSelectButtonImage;
		Gtk::Button ipFilterFileSelectButton;
		Gtk::HBox ipFilterFileRow;

		// Ip TreeView
		Gtk::Frame ipFilterFrame;
		class ModelColumnsIpFilter : public Gtk::TreeModel::ColumnRecord{
		public:
			ModelColumnsIpFilter(){
				add(m_first_ip); add(m_last_ip);
			}
			Gtk::TreeModelColumn<Glib::ustring> m_first_ip;
			Gtk::TreeModelColumn<Glib::ustring> m_last_ip;
		};

		Gtk::ScrolledWindow ipFilterWindow;
		Gtk::TreeView ipFilterTreeView;
		Glib::RefPtr<Gtk::ListStore> refTreeModel;
		ModelColumnsIpFilter ipFilterColumns;

		// Buttons
		Gtk::Button addIpButton, removeIpButton;
		Gtk::Alignment spacer;
		Gtk::HBox buttonsHBox;

		// Signals
		void setIpFilterWindowSensivity();

	public:
		IpFilterWindow(OptionsTab *);
		~IpFilterWindow();

		// Get options
		bool getActivateIpFilterCheckButton() { return activateIpFilterCheckButton.get_active(); };

		// Set options
		void setActivateIpFilterCheckButton(bool value) { activateIpFilterCheckButton.set_active(value); };

		bool isRowSelected();
		void updateIpFilterData();
		Glib::ustring getFirstSelectedIp();
		Glib::ustring getLastSelectedIp();
};

class SearchWindow: public OptionsWindowsBase, public Gtk::ScrolledWindow
{
	private:
		// Global vbox
		Gtk::VBox vbox;
		Gtk::CheckButton btjunkieCheckButton, isohuntCheckButton, piratebayCheckButton;
		Gtk::CheckButton mininovaCheckButton, torrentreactorCheckButton;
		Gtk::CheckButton filterResultsWithoutSeeders;
		Gtk::SpinButton maxSearches;

	public:
		SearchWindow(OptionsTab *);
		~SearchWindow();

		// Get options
		bool getBtjunkieCheckButton() { return btjunkieCheckButton.get_active(); };
		bool getIsohuntCheckButton() { return isohuntCheckButton.get_active(); };
		bool getPiratebayCheckButton() { return piratebayCheckButton.get_active(); };
		bool getMininovaCheckButton() { return mininovaCheckButton.get_active(); };
		bool getTorrentreactorCheckButton() { return torrentreactorCheckButton.get_active(); };
		bool getFilterSeedersCheckButton() { return filterResultsWithoutSeeders.get_active(); };
		int getMaxSearchesValue() { return maxSearches.get_value_as_int(); };

		// Set options
		void setBtjunkieCheckButton(bool value) { btjunkieCheckButton.set_active(value); };
		void setIsohuntCheckButton(bool value) { isohuntCheckButton.set_active(value); };
		void setPiratebayCheckButton(bool value) { piratebayCheckButton.set_active(value); };
		void setMininovaCheckButton(bool value) { mininovaCheckButton.set_active(value); };
		void setTorrentreactorCheckButton(bool value) { torrentreactorCheckButton.set_active(value); };
		void setFilterSeedersCheckButton(bool value) { filterResultsWithoutSeeders.set_active(value); };
		void setMaxSearchesValue(int value) { maxSearches.set_value((double) value); };
};

class SchedulerWindow: public OptionsWindowsBase, public Gtk::ScrolledWindow
{
	private:
		// Global vbox
		Gtk::VBox vbox;
		Gtk::CheckButton useSchedulerCheckButton;

		Gtk::Frame scheduler_frame;

		Gtk::SpinButton downloadHour0, uploadHour0, downloadHour1, uploadHour1;
		Gtk::SpinButton downloadHour2, uploadHour2, downloadHour3, uploadHour3;
		Gtk::SpinButton downloadHour4, uploadHour4, downloadHour5, uploadHour5;
		Gtk::SpinButton downloadHour6, uploadHour6, downloadHour7, uploadHour7;
		Gtk::SpinButton downloadHour8, uploadHour8, downloadHour9, uploadHour9;
		Gtk::SpinButton downloadHour10, uploadHour10, downloadHour11, uploadHour11;
		Gtk::SpinButton downloadHour12, uploadHour12, downloadHour13, uploadHour13;
		Gtk::SpinButton downloadHour14, uploadHour14, downloadHour15, uploadHour15;
		Gtk::SpinButton downloadHour16, uploadHour16, downloadHour17, uploadHour17;
		Gtk::SpinButton downloadHour18, uploadHour18, downloadHour19, uploadHour19;
		Gtk::SpinButton downloadHour20, uploadHour20, downloadHour21, uploadHour21;
		Gtk::SpinButton downloadHour22, uploadHour22, downloadHour23, uploadHour23;

		Gtk::CheckButton monday, tuesday, wednesday, thursday, friday;
		Gtk::CheckButton saturday, sunday;

		// Signals
		void setFramesSensivity();

	public:
		SchedulerWindow(OptionsTab *);
		~SchedulerWindow();

		bool getUseSchedulerCheckButton() { return useSchedulerCheckButton.get_active(); };

		bool getMondayCheckButton() { return monday.get_active(); };
		bool getTuesdayCheckButon() { return tuesday.get_active(); };
		bool getWednesdayCheckButton() { return wednesday.get_active(); };
		bool getThursdayCheckButton() { return thursday.get_active(); };
		bool getFridayCheckButton() { return friday.get_active(); };
		bool getSaturdayCheckButton() { return saturday.get_active(); };
		bool getSundayCheckButton() { return sunday.get_active(); };

		int getDownloadHour0Value() { return downloadHour0.get_value_as_int(); };
		int getUploadHour0Value() { return uploadHour0.get_value_as_int(); };
		int getDownloadHour1Value() { return downloadHour1.get_value_as_int(); };
		int getUploadHour1Value() { return uploadHour1.get_value_as_int(); };
		int getDownloadHour2Value() { return downloadHour2.get_value_as_int(); };
		int getUploadHour2Value() { return uploadHour2.get_value_as_int(); };
		int getDownloadHour3Value() { return downloadHour3.get_value_as_int(); };
		int getUploadHour3Value() { return uploadHour3.get_value_as_int(); };
		int getDownloadHour4Value() { return downloadHour4.get_value_as_int(); };
		int getUploadHour4Value() { return uploadHour4.get_value_as_int(); };
		int getDownloadHour5Value() { return downloadHour5.get_value_as_int(); };
		int getUploadHour5Value() { return uploadHour5.get_value_as_int(); };
		int getDownloadHour6Value() { return downloadHour6.get_value_as_int(); };
		int getUploadHour6Value() { return uploadHour6.get_value_as_int(); };
		int getDownloadHour7Value() { return downloadHour7.get_value_as_int(); };
		int getUploadHour7Value() { return uploadHour7.get_value_as_int(); };
		int getDownloadHour8Value() { return downloadHour8.get_value_as_int(); };
		int getUploadHour8Value() { return uploadHour8.get_value_as_int(); };
		int getDownloadHour9Value() { return downloadHour9.get_value_as_int(); };
		int getUploadHour9Value() { return uploadHour9.get_value_as_int(); };
		int getDownloadHour10Value() { return downloadHour10.get_value_as_int(); };
		int getUploadHour10Value() { return uploadHour10.get_value_as_int(); };
		int getDownloadHour11Value() { return downloadHour11.get_value_as_int(); };
		int getUploadHour11Value() { return uploadHour11.get_value_as_int(); };
		int getDownloadHour12Value() { return downloadHour12.get_value_as_int(); };
		int getUploadHour12Value() { return uploadHour12.get_value_as_int(); };
		int getDownloadHour13Value() { return downloadHour13.get_value_as_int(); };
		int getUploadHour13Value() { return uploadHour13.get_value_as_int(); };
		int getDownloadHour14Value() { return downloadHour14.get_value_as_int(); };
		int getUploadHour14Value() { return uploadHour14.get_value_as_int(); };
		int getDownloadHour15Value() { return downloadHour15.get_value_as_int(); };
		int getUploadHour15Value() { return uploadHour15.get_value_as_int(); };
		int getDownloadHour16Value() { return downloadHour16.get_value_as_int(); };
		int getUploadHour16Value() { return uploadHour16.get_value_as_int(); };
		int getDownloadHour17Value() { return downloadHour17.get_value_as_int(); };
		int getUploadHour17Value() { return uploadHour17.get_value_as_int(); };
		int getDownloadHour18Value() { return downloadHour18.get_value_as_int(); };
		int getUploadHour18Value() { return uploadHour18.get_value_as_int(); };
		int getDownloadHour19Value() { return downloadHour19.get_value_as_int(); };
		int getUploadHour19Value() { return uploadHour19.get_value_as_int(); };
		int getDownloadHour20Value() { return downloadHour20.get_value_as_int(); };
		int getUploadHour20Value() { return uploadHour20.get_value_as_int(); };
		int getDownloadHour21Value() { return downloadHour21.get_value_as_int(); };
		int getUploadHour21Value() { return uploadHour21.get_value_as_int(); };
		int getDownloadHour22Value() { return downloadHour22.get_value_as_int(); };
		int getUploadHour22Value() { return uploadHour22.get_value_as_int(); };
		int getDownloadHour23Value() { return downloadHour23.get_value_as_int(); };
		int getUploadHour23Value() { return uploadHour23.get_value_as_int(); };

		void setUseSchedulerCheckButton(bool value) { useSchedulerCheckButton.set_active(value); };

		void setMondayCheckButton(bool value) { monday.set_active(value); };
		void setTuesdayCheckButton(bool value) { tuesday.set_active(value); };
		void setWednesdayCheckButton(bool value) { wednesday.set_active(value); };
		void setThursdayCheckButton(bool value) { thursday.set_active(value); };
		void setFridayCheckButton(bool value) { friday.set_active(value); };
		void setSaturdayCheckButton(bool value) { saturday.set_active(value); };
		void setSundayCheckButton(bool value) { sunday.set_active(value); };

		void setDownloadHour0Value(int value) { downloadHour0.set_value((double) value); };
		void setUploadHour0Value(int value) { uploadHour0.set_value((double) value); };
		void setDownloadHour1Value(int value) { downloadHour1.set_value((double) value); };
		void setUploadHour1Value(int value) { uploadHour1.set_value((double) value); };
		void setDownloadHour2Value(int value) { downloadHour2.set_value((double) value); };
		void setUploadHour2Value(int value) { uploadHour2.set_value((double) value); };
		void setDownloadHour3Value(int value) { downloadHour3.set_value((double) value); };
		void setUploadHour3Value(int value) { uploadHour3.set_value((double) value); };
		void setDownloadHour4Value(int value) { downloadHour4.set_value((double) value); };
		void setUploadHour4Value(int value) { uploadHour4.set_value((double) value); };
		void setDownloadHour5Value(int value) { downloadHour5.set_value((double) value); };
		void setUploadHour5Value(int value) { uploadHour5.set_value((double) value); };
		void setDownloadHour6Value(int value) { downloadHour6.set_value((double) value); };
		void setUploadHour6Value(int value) { uploadHour6.set_value((double) value); };
		void setDownloadHour7Value(int value) { downloadHour7.set_value((double) value); };
		void setUploadHour7Value(int value) { uploadHour7.set_value((double) value); };
		void setDownloadHour8Value(int value) { downloadHour8.set_value((double) value); };
		void setUploadHour8Value(int value) { uploadHour8.set_value((double) value); };
		void setDownloadHour9Value(int value) { downloadHour9.set_value((double) value); };
		void setUploadHour9Value(int value) { uploadHour9.set_value((double) value); };
		void setDownloadHour10Value(int value) { downloadHour10.set_value((double) value); };
		void setUploadHour10Value(int value) { uploadHour10.set_value((double) value); };
		void setDownloadHour11Value(int value) { downloadHour11.set_value((double) value); };
		void setUploadHour11Value(int value) { uploadHour11.set_value((double) value); };
		void setDownloadHour12Value(int value) { downloadHour12.set_value((double) value); };
		void setUploadHour12Value(int value) { uploadHour12.set_value((double) value); };
		void setDownloadHour13Value(int value) { downloadHour13.set_value((double) value); };
		void setUploadHour13Value(int value) { uploadHour13.set_value((double) value); };
		void setDownloadHour14Value(int value) { downloadHour14.set_value((double) value); };
		void setUploadHour14Value(int value) { uploadHour14.set_value((double) value); };
		void setDownloadHour15Value(int value) { downloadHour15.set_value((double) value); };
		void setUploadHour15Value(int value) { uploadHour15.set_value((double) value); };
		void setDownloadHour16Value(int value) { downloadHour16.set_value((double) value); };
		void setUploadHour16Value(int value) { uploadHour16.set_value((double) value); };
		void setDownloadHour17Value(int value) { downloadHour17.set_value((double) value); };
		void setUploadHour17Value(int value) { uploadHour17.set_value((double) value); };
		void setDownloadHour18Value(int value) { downloadHour18.set_value((double) value); };
		void setUploadHour18Value(int value) { uploadHour18.set_value((double) value); };
		void setDownloadHour19Value(int value) { downloadHour19.set_value((double) value); };
		void setUploadHour19Value(int value) { uploadHour19.set_value((double) value); };
		void setDownloadHour20Value(int value) { downloadHour20.set_value((double) value); };
		void setUploadHour20Value(int value) { uploadHour20.set_value((double) value); };
		void setDownloadHour21Value(int value) { downloadHour21.set_value((double) value); };
		void setUploadHour21Value(int value) { uploadHour21.set_value((double) value); };
		void setDownloadHour22Value(int value) { downloadHour22.set_value((double) value); };
		void setUploadHour22Value(int value) { uploadHour22.set_value((double) value); };
		void setDownloadHour23Value(int value) { downloadHour23.set_value((double) value); };
		void setUploadHour23Value(int value) { uploadHour23.set_value((double) value); };
};

#endif /* _OPTIONSWINDOWS_H */
