/***************************************************************************
 *            PiecesInfoNoteBookTab.cc
 *
 *  Mon Sep 10 00:54:16 2007
 *  Copyright  2007  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA

#include "PiecesInfoNoteBookTab.h"

#include "../config.h"
#include <glibmm/i18n.h>

using namespace Lince;

PiecesInfoNoteBookTab::PiecesInfoNoteBookTab():
	nameTab(_("Pieces"))
{
	// Add the TreeView, inside the ScrolledWindow, with the button underneath:
	// Only show the scrollbars when they are necessary:
	add(piecesTreeView);
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
		
	// Create the Tree model:
	refTreeModel = Gtk::ListStore::create(piecesColumns);
	piecesTreeView.set_model(refTreeModel);
	piecesTreeView.set_rules_hint();
	
	// Add the TreeView's view columns:
	piecesTreeView.append_column(_("Number"), piecesColumns.m_piece_num);
		
	Gtk::CellRendererProgress* cell = Gtk::manage(new Gtk::CellRendererProgress);
	int cols_count = piecesTreeView.append_column(_("Percentage"), *cell);
	Gtk::TreeViewColumn* pColumn = piecesTreeView.get_column(cols_count-1);
	if(pColumn)
	{
#ifdef GLIBMM_PROPERTIES_ENABLED
				pColumn->add_attribute(cell->property_value(), piecesColumns.m_piece_percentage);
#else
				pColumn->add_attribute(*cell, _("value"), piecesColumns.m_piece_percentage);
#endif
	}
	
	for(guint i = 0; i < 2; i++)
	{
		Gtk::TreeView::Column* pColumn = piecesTreeView.get_column(i);
		Gtk::CellRenderer* cColumn = piecesTreeView.get_column_cell_renderer(i);
		
		//pColumn->set_reorderable();		
		pColumn->set_resizable();
		
		switch(i)
		{
			case 0:
				pColumn->set_sort_column(piecesColumns.m_piece_num);
				break;
			case 1:
				pColumn->set_sort_column(piecesColumns.m_piece_percentage);
				pColumn->set_expand();
				pColumn->set_alignment(0.5);
				cColumn->property_xalign() = 0.0;
				break;
		}
	}
	
	piecesTreeView.property_can_focus() = false;
	piecesTreeView.signal_cursor_changed().connect( sigc::mem_fun(this, &PiecesInfoNoteBookTab::unselectRows) );
	piecesTreeView.columns_autosize();
}

void PiecesInfoNoteBookTab::updatePiecesInfoNoteBookTab(Torrent *t)
{
	std::vector<libtorrent::partial_piece_info> queue;
	Gtk::TreeModel::Row row;
	
	refTreeModel->clear();	
	t->get_download_queue(queue);
	for (std::vector<libtorrent::partial_piece_info>::iterator i = queue.begin(); i != queue.end(); ++i)
	{
		row = *(refTreeModel->append());
		row[piecesColumns.m_piece_num] = i->piece_index;
		//row[piecesColumns.m_piece_percentage] = 0;
		
		int finished_blocks = 0;
		
		for (int j=0; j < i->blocks_in_piece; j++)
		{
			if (i->blocks[j].state == 2 || i->blocks[j].state == 3)
				finished_blocks++;
		}
		
		//row[piecesColumns.m_piece_percentage] = (i->finished_blocks.count() * 100) / i->blocks_in_piece;
		row[piecesColumns.m_piece_percentage] = (finished_blocks * 100) / i->blocks_in_piece;
	}
}

void PiecesInfoNoteBookTab::unselectRows()
{
	Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = piecesTreeView.get_selection();
	
	refTreeSelection->unselect_all();
}

PiecesInfoNoteBookTab::~PiecesInfoNoteBookTab()
{
}
