/***************************************************************************
 *            TorrentsTab.h
 *
 *  Sat Feb 17 21:43:21 2007
 *  Copyright  2007  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */
 
#ifndef _TORRENTSTAB_H
#define _TORRENTSTAB_H

#include <gtkmm.h>

#include "libtorrent/session.hpp"

#include "TorrentsWindow.h"
#include "BaseTab.h"
#include "Utils.h"

#include "Lince/Torrent.h"
#include "Lince/TorrentsManager.h"

class MainWindow;
class MenuBar;

class TorrentsTab: public BaseTab{
	private:
		// Panes.
		Gtk::VPaned pane;
	
		// Downloading Torrents Window
		TorrentsWindow downloadingTorrents;
		
		// Downloaded Torrents Window
		TorrentsWindow downloadedTorrents;
		
		void addTorrentFromPosition(Lince::torrentPosition);
		void switchTorrent(int);
	
	public:
		TorrentsTab(MainWindow *, MenuBar *);
		~TorrentsTab();
	
		Lince::torrentPosition getSelectedTorrentPosition();
		std::vector<Lince::torrentPosition> getSelectedTorrentsPositions();
		
		int getDownloadingSelectedTorrents(void) { return downloadingTorrents.torrentsSelected(); };
		int getDownloadingSelectedRow() { return downloadingTorrents.getSelectedRow(); };
		int getDownloadedSelectedTorrents(void) { return downloadedTorrents.torrentsSelected(); };
		int getDownloadedSelectedRow() { return downloadedTorrents.getSelectedRow(); };		
	
		bool isDownloadingSorted(void) { return downloadingTorrents.isSorted(); };
		bool isDownloadedSorted(void) { return downloadedTorrents.isSorted(); };
	
		void setSelectedDownloadingRow(int tId) { downloadingTorrents.setSelectedRow(tId); };
		void setSelectedDownloadedRow(int tId) { downloadedTorrents.setSelectedRow(tId); };
	
		void removeDownloadingRowGivenTID(int tId) { downloadingTorrents.removeGivenRow(tId); };
		void removeDownloadedRowGivenTID(int tId) { downloadedTorrents.removeGivenRow(tId); };
						
		void addTorrent(Lince::Torrent *t) { downloadingTorrents.addRow(t); };
		void addFinishedTorrent(Lince::Torrent *t) { downloadedTorrents.addRow(t); };
		
		void updateDownloadingTorrents() { downloadingTorrents.updateRows(); };
		void updateDownloadedTorrents() { downloadedTorrents.updateRows(); };
	
		void autoSizeDownloadingTorrentsColumns(void) { downloadingTorrents.autoSizeColums(); };
		void autoSizeDownloadedTorrentsColumns(void) { downloadedTorrents.autoSizeColums(); };
	
		void clearTorrentsList() { downloadingTorrents.clearRows(); downloadedTorrents.clearRows(); };
	
		void checkTorrentsSelection(TorrentsWindowType);
		
		virtual Gtk::Widget * getTabChild(void) { return & pane; };
};
		

#endif /* _TORRENTSTAB_H */

 
