/***************************************************************************
 *            XMLReader.cc
 *
 *  Wed Jan  3 17:08:02 2007
 *  Copyright  2007  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */
 
#include "XMLReader.h"

XMLReader::XMLReader(){
	LIBXML_TEST_VERSION
	
	numattrs = currentattr = 0;
	reader   = NULL;
			
	presentnode = filename = childdata = "";
	attributename = attributedata = presentnode = "";
}

/*!
 This constructor sets the file name which will be used to retrieve or to 
 keep the xml data.
*/
XMLReader::XMLReader(const Glib::ustring &fname){
	LIBXML_TEST_VERSION	
	
	numattrs = currentattr = 0;
	reader   = NULL;
				
	filename = fname;
	presentnode = childdata = attributename = "";
	attributedata = presentnode = "";
}

XMLReader::~XMLReader(){
	if (isReaderInitilizated()){
		freeReader();
	}
	
	xmlCleanupParser();
}

bool XMLReader::initializeReader(){
	if (!filename.length()){
		return false;
	}
	
	reader = xmlReaderForFile(filename.c_str(), NULL, 0);
	
	if (reader) return true;
		
	return false;
}

bool XMLReader::goToRootElement(){
	if (isReaderInitilizated()){
		freeReader();
	}
	
	initializeReader();
	if (!isReaderInitilizated()){
		return false;
	}		
	
	if (goToNextElement() != 1) return false;
		
	return true;
}

int XMLReader::goToNextElement(){
	int ret, temp_depth;
	Glib::ustring temp_father;
	const xmlChar *name, *child_data;
	xmlNodePtr child_node;
	
	if (!isReaderInitilizated()){
		return -1;
	}
	
	temp_depth = getDepth();
	temp_father = getNodeName();
	
	do{
		ret = xmlTextReaderRead(reader);
	}while((xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT) && ret == 1);
	
	if (temp_depth > MAXVALUE || numattrs > MAXVALUE){
		return -1;
	}
	
	if (temp_depth < getDepth()){
		father_node[temp_depth] = temp_father;
	}
	
	numattrs = xmlTextReaderAttributeCount(reader);
	currentattr = 0;
	if (numattrs){
		for (int i=0; i<numattrs; i++){
			name = xmlTextReaderGetAttributeNo(reader, i);
			attributedatatmp[i] = (const char *) name;
			xmlFree((void *) name);
		}
	}
	
	// Procesar nodo
	name = xmlTextReaderConstName(reader);
    if (name == NULL) name = BAD_CAST "";
		
	presentnode = (const char *) name;
			
	child_node = xmlTextReaderExpand(reader);
	child_data = xmlNodeGetContent(child_node);
	
	if (child_data){
		childdata = (const char *) child_data;
		xmlFree((void *) child_data);
	}
	else childdata = "";
		
	return ret;
}

bool XMLReader::findChildFromBeginning(const Glib::ustring &name){
	goToRootElement();	
	return findChild(name);
}

bool XMLReader::findChild(const Glib::ustring &name){
	int ret;
	const xmlChar *find_name = (const xmlChar*) name.c_str(), *tmp_name;
	
	if (!isReaderInitilizated()){
		goToRootElement();
	}
	
	// Buscar en cada nodo	
	do{
		tmp_name = xmlTextReaderConstName(reader);
		if (!xmlStrcmp(find_name, tmp_name)){
			return true;
		}
	}while((ret = goToNextElement()) == 1);
			
	
	// No encontrado o fichero mal formado
		
	// Liberar recursos
	freeReader();
	return false;
}

Glib::ustring XMLReader::getChildData(){
	Glib::ustring rstr("");
	
	if (childdata.size())
		rstr = childdata;
	
	if (childdata.find("\n") != childdata.rfind("\n"))
		rstr = "";
	
	return rstr;
}

/*!
 Used to retrieve the attribute data of the element.
*/
bool XMLReader::goToNextAttribute(){
	int ret=1;
	const xmlChar *attr_name;
	
	
	if ( currentattr < numattrs ){
		ret = xmlTextReaderMoveToNextAttribute(reader);
		attr_name = xmlTextReaderConstName(reader);
				
		if (ret != 1 || attr_name == NULL){
			return false;
		}
		
		attributename = (const char *) attr_name;
		attributedata = attributedatatmp[currentattr];
		
		currentattr += 1;
		
		return true;
	}
	
	return false;
}
